// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "content/common/service_worker/service_worker_provider.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "content/common/service_worker/service_worker_provider.mojom-shared-message-ids.h"
namespace content {
namespace mojom {
const char kNavigation_ServiceWorkerSpec[] = "navigation:service_worker";
ServiceWorkerProviderInfoForSharedWorker::ServiceWorkerProviderInfoForSharedWorker()
    : provider_id(),
      host_ptr_info(),
      client_request() {}

ServiceWorkerProviderInfoForSharedWorker::ServiceWorkerProviderInfoForSharedWorker(
    int32_t provider_id_in,
    ::content::mojom::ServiceWorkerContainerHostAssociatedPtrInfo host_ptr_info_in,
    ::content::mojom::ServiceWorkerContainerAssociatedRequest client_request_in)
    : provider_id(std::move(provider_id_in)),
      host_ptr_info(std::move(host_ptr_info_in)),
      client_request(std::move(client_request_in)) {}

ServiceWorkerProviderInfoForSharedWorker::~ServiceWorkerProviderInfoForSharedWorker() = default;

bool ServiceWorkerProviderInfoForSharedWorker::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
ServiceWorkerProviderInfoForStartWorker::ServiceWorkerProviderInfoForStartWorker()
    : provider_id(),
      host_ptr_info(),
      client_request(),
      script_loader_factory_ptr_info(),
      cache_storage(),
      interface_provider() {}

ServiceWorkerProviderInfoForStartWorker::ServiceWorkerProviderInfoForStartWorker(
    int32_t provider_id_in,
    ::content::mojom::ServiceWorkerContainerHostAssociatedPtrInfo host_ptr_info_in,
    ::content::mojom::ServiceWorkerContainerAssociatedRequest client_request_in,
    ::network::mojom::URLLoaderFactoryAssociatedPtrInfo script_loader_factory_ptr_info_in,
    ::blink::mojom::CacheStoragePtrInfo cache_storage_in,
    ::service_manager::mojom::InterfaceProviderPtrInfo interface_provider_in)
    : provider_id(std::move(provider_id_in)),
      host_ptr_info(std::move(host_ptr_info_in)),
      client_request(std::move(client_request_in)),
      script_loader_factory_ptr_info(std::move(script_loader_factory_ptr_info_in)),
      cache_storage(std::move(cache_storage_in)),
      interface_provider(std::move(interface_provider_in)) {}

ServiceWorkerProviderInfoForStartWorker::~ServiceWorkerProviderInfoForStartWorker() = default;

bool ServiceWorkerProviderInfoForStartWorker::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
ServiceWorkerProviderHostInfo::ServiceWorkerProviderHostInfo()
    : provider_id(),
      route_id(),
      type(),
      is_parent_frame_secure(),
      host_request(),
      client_ptr_info() {}

ServiceWorkerProviderHostInfo::ServiceWorkerProviderHostInfo(
    int32_t provider_id_in,
    int32_t route_id_in,
    ::blink::mojom::ServiceWorkerProviderType type_in,
    bool is_parent_frame_secure_in,
    ::content::mojom::ServiceWorkerContainerHostAssociatedRequest host_request_in,
    ::content::mojom::ServiceWorkerContainerAssociatedPtrInfo client_ptr_info_in)
    : provider_id(std::move(provider_id_in)),
      route_id(std::move(route_id_in)),
      type(std::move(type_in)),
      is_parent_frame_secure(std::move(is_parent_frame_secure_in)),
      host_request(std::move(host_request_in)),
      client_ptr_info(std::move(client_ptr_info_in)) {}

ServiceWorkerProviderHostInfo::~ServiceWorkerProviderHostInfo() = default;

bool ServiceWorkerProviderHostInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char ServiceWorkerWorkerClient::Name_[] = "content.mojom.ServiceWorkerWorkerClient";

ServiceWorkerWorkerClientProxy::ServiceWorkerWorkerClientProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void ServiceWorkerWorkerClientProxy::OnControllerChanged(
    ::blink::mojom::ControllerServiceWorkerMode in_mode) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "content::mojom::ServiceWorkerWorkerClient::OnControllerChanged");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kServiceWorkerWorkerClient_OnControllerChanged_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::content::mojom::internal::ServiceWorkerWorkerClient_OnControllerChanged_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::blink::mojom::ControllerServiceWorkerMode>(
      in_mode, &params->mode);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(ServiceWorkerWorkerClient::Name_);
  message.set_method_name("OnControllerChanged");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool ServiceWorkerWorkerClientStubDispatch::Accept(
    ServiceWorkerWorkerClient* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kServiceWorkerWorkerClient_OnControllerChanged_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)content::mojom::ServiceWorkerWorkerClient::OnControllerChanged",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::ServiceWorkerWorkerClient_OnControllerChanged_Params_Data* params =
          reinterpret_cast<internal::ServiceWorkerWorkerClient_OnControllerChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::blink::mojom::ControllerServiceWorkerMode p_mode{};
      ServiceWorkerWorkerClient_OnControllerChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadMode(&p_mode))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "ServiceWorkerWorkerClient::OnControllerChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnControllerChanged(
std::move(p_mode));
      return true;
    }
  }
  return false;
}

// static
bool ServiceWorkerWorkerClientStubDispatch::AcceptWithResponder(
    ServiceWorkerWorkerClient* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kServiceWorkerWorkerClient_OnControllerChanged_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)content::mojom::ServiceWorkerWorkerClient::OnControllerChanged",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool ServiceWorkerWorkerClientRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "ServiceWorkerWorkerClient RequestValidator");

  switch (message->header()->name) {
    case internal::kServiceWorkerWorkerClient_OnControllerChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::ServiceWorkerWorkerClient_OnControllerChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void ServiceWorkerWorkerClientInterceptorForTesting::OnControllerChanged(::blink::mojom::ControllerServiceWorkerMode mode) {
  GetForwardingInterface()->OnControllerChanged(std::move(mode));
}
ServiceWorkerWorkerClientAsyncWaiter::ServiceWorkerWorkerClientAsyncWaiter(
    ServiceWorkerWorkerClient* proxy) : proxy_(proxy) {}

ServiceWorkerWorkerClientAsyncWaiter::~ServiceWorkerWorkerClientAsyncWaiter() = default;


const char ServiceWorkerWorkerClientRegistry::Name_[] = "content.mojom.ServiceWorkerWorkerClientRegistry";

ServiceWorkerWorkerClientRegistryProxy::ServiceWorkerWorkerClientRegistryProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void ServiceWorkerWorkerClientRegistryProxy::RegisterWorkerClient(
    ServiceWorkerWorkerClientPtr in_client) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "content::mojom::ServiceWorkerWorkerClientRegistry::RegisterWorkerClient");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kServiceWorkerWorkerClientRegistry_RegisterWorkerClient_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::content::mojom::internal::ServiceWorkerWorkerClientRegistry_RegisterWorkerClient_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::content::mojom::ServiceWorkerWorkerClientPtrDataView>(
      in_client, &params->client, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid client in ServiceWorkerWorkerClientRegistry.RegisterWorkerClient request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(ServiceWorkerWorkerClientRegistry::Name_);
  message.set_method_name("RegisterWorkerClient");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void ServiceWorkerWorkerClientRegistryProxy::CloneWorkerClientRegistry(
    ServiceWorkerWorkerClientRegistryRequest in_host) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "content::mojom::ServiceWorkerWorkerClientRegistry::CloneWorkerClientRegistry");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kServiceWorkerWorkerClientRegistry_CloneWorkerClientRegistry_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::content::mojom::internal::ServiceWorkerWorkerClientRegistry_CloneWorkerClientRegistry_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::content::mojom::ServiceWorkerWorkerClientRegistryRequestDataView>(
      in_host, &params->host, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->host),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid host in ServiceWorkerWorkerClientRegistry.CloneWorkerClientRegistry request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(ServiceWorkerWorkerClientRegistry::Name_);
  message.set_method_name("CloneWorkerClientRegistry");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool ServiceWorkerWorkerClientRegistryStubDispatch::Accept(
    ServiceWorkerWorkerClientRegistry* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kServiceWorkerWorkerClientRegistry_RegisterWorkerClient_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)content::mojom::ServiceWorkerWorkerClientRegistry::RegisterWorkerClient",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::ServiceWorkerWorkerClientRegistry_RegisterWorkerClient_Params_Data* params =
          reinterpret_cast<internal::ServiceWorkerWorkerClientRegistry_RegisterWorkerClient_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ServiceWorkerWorkerClientPtr p_client{};
      ServiceWorkerWorkerClientRegistry_RegisterWorkerClient_ParamsDataView input_data_view(params, &serialization_context);
      
      p_client =
          input_data_view.TakeClient<decltype(p_client)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "ServiceWorkerWorkerClientRegistry::RegisterWorkerClient deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->RegisterWorkerClient(
std::move(p_client));
      return true;
    }
    case internal::kServiceWorkerWorkerClientRegistry_CloneWorkerClientRegistry_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)content::mojom::ServiceWorkerWorkerClientRegistry::CloneWorkerClientRegistry",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::ServiceWorkerWorkerClientRegistry_CloneWorkerClientRegistry_Params_Data* params =
          reinterpret_cast<internal::ServiceWorkerWorkerClientRegistry_CloneWorkerClientRegistry_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ServiceWorkerWorkerClientRegistryRequest p_host{};
      ServiceWorkerWorkerClientRegistry_CloneWorkerClientRegistry_ParamsDataView input_data_view(params, &serialization_context);
      
      p_host =
          input_data_view.TakeHost<decltype(p_host)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "ServiceWorkerWorkerClientRegistry::CloneWorkerClientRegistry deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CloneWorkerClientRegistry(
std::move(p_host));
      return true;
    }
  }
  return false;
}

// static
bool ServiceWorkerWorkerClientRegistryStubDispatch::AcceptWithResponder(
    ServiceWorkerWorkerClientRegistry* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kServiceWorkerWorkerClientRegistry_RegisterWorkerClient_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)content::mojom::ServiceWorkerWorkerClientRegistry::RegisterWorkerClient",
               "message", message->name());
#endif
      break;
    }
    case internal::kServiceWorkerWorkerClientRegistry_CloneWorkerClientRegistry_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)content::mojom::ServiceWorkerWorkerClientRegistry::CloneWorkerClientRegistry",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool ServiceWorkerWorkerClientRegistryRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "ServiceWorkerWorkerClientRegistry RequestValidator");

  switch (message->header()->name) {
    case internal::kServiceWorkerWorkerClientRegistry_RegisterWorkerClient_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::ServiceWorkerWorkerClientRegistry_RegisterWorkerClient_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kServiceWorkerWorkerClientRegistry_CloneWorkerClientRegistry_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::ServiceWorkerWorkerClientRegistry_CloneWorkerClientRegistry_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void ServiceWorkerWorkerClientRegistryInterceptorForTesting::RegisterWorkerClient(ServiceWorkerWorkerClientPtr client) {
  GetForwardingInterface()->RegisterWorkerClient(std::move(client));
}
void ServiceWorkerWorkerClientRegistryInterceptorForTesting::CloneWorkerClientRegistry(ServiceWorkerWorkerClientRegistryRequest host) {
  GetForwardingInterface()->CloneWorkerClientRegistry(std::move(host));
}
ServiceWorkerWorkerClientRegistryAsyncWaiter::ServiceWorkerWorkerClientRegistryAsyncWaiter(
    ServiceWorkerWorkerClientRegistry* proxy) : proxy_(proxy) {}

ServiceWorkerWorkerClientRegistryAsyncWaiter::~ServiceWorkerWorkerClientRegistryAsyncWaiter() = default;


const char ServiceWorkerDispatcherHost::Name_[] = "content.mojom.ServiceWorkerDispatcherHost";

ServiceWorkerDispatcherHostProxy::ServiceWorkerDispatcherHostProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void ServiceWorkerDispatcherHostProxy::OnProviderCreated(
    ServiceWorkerProviderHostInfoPtr in_provider_info) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "content::mojom::ServiceWorkerDispatcherHost::OnProviderCreated");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kServiceWorkerDispatcherHost_OnProviderCreated_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::content::mojom::internal::ServiceWorkerDispatcherHost_OnProviderCreated_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->provider_info)::BaseType::BufferWriter
      provider_info_writer;
  mojo::internal::Serialize<::content::mojom::ServiceWorkerProviderHostInfoDataView>(
      in_provider_info, buffer, &provider_info_writer, &serialization_context);
  params->provider_info.Set(
      provider_info_writer.is_null() ? nullptr : provider_info_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->provider_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null provider_info in ServiceWorkerDispatcherHost.OnProviderCreated request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(ServiceWorkerDispatcherHost::Name_);
  message.set_method_name("OnProviderCreated");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool ServiceWorkerDispatcherHostStubDispatch::Accept(
    ServiceWorkerDispatcherHost* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kServiceWorkerDispatcherHost_OnProviderCreated_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)content::mojom::ServiceWorkerDispatcherHost::OnProviderCreated",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::ServiceWorkerDispatcherHost_OnProviderCreated_Params_Data* params =
          reinterpret_cast<internal::ServiceWorkerDispatcherHost_OnProviderCreated_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ServiceWorkerProviderHostInfoPtr p_provider_info{};
      ServiceWorkerDispatcherHost_OnProviderCreated_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadProviderInfo(&p_provider_info))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "ServiceWorkerDispatcherHost::OnProviderCreated deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnProviderCreated(
std::move(p_provider_info));
      return true;
    }
  }
  return false;
}

// static
bool ServiceWorkerDispatcherHostStubDispatch::AcceptWithResponder(
    ServiceWorkerDispatcherHost* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kServiceWorkerDispatcherHost_OnProviderCreated_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)content::mojom::ServiceWorkerDispatcherHost::OnProviderCreated",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool ServiceWorkerDispatcherHostRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "ServiceWorkerDispatcherHost RequestValidator");

  switch (message->header()->name) {
    case internal::kServiceWorkerDispatcherHost_OnProviderCreated_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::ServiceWorkerDispatcherHost_OnProviderCreated_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void ServiceWorkerDispatcherHostInterceptorForTesting::OnProviderCreated(ServiceWorkerProviderHostInfoPtr provider_info) {
  GetForwardingInterface()->OnProviderCreated(std::move(provider_info));
}
ServiceWorkerDispatcherHostAsyncWaiter::ServiceWorkerDispatcherHostAsyncWaiter(
    ServiceWorkerDispatcherHost* proxy) : proxy_(proxy) {}

ServiceWorkerDispatcherHostAsyncWaiter::~ServiceWorkerDispatcherHostAsyncWaiter() = default;


}  // namespace mojom
}  // namespace content

namespace mojo {


// static
bool StructTraits<::content::mojom::ServiceWorkerProviderInfoForSharedWorker::DataView, ::content::mojom::ServiceWorkerProviderInfoForSharedWorkerPtr>::Read(
    ::content::mojom::ServiceWorkerProviderInfoForSharedWorker::DataView input,
    ::content::mojom::ServiceWorkerProviderInfoForSharedWorkerPtr* output) {
  bool success = true;
  ::content::mojom::ServiceWorkerProviderInfoForSharedWorkerPtr result(::content::mojom::ServiceWorkerProviderInfoForSharedWorker::New());
  
      result->provider_id = input.provider_id();
      result->host_ptr_info =
          input.TakeHostPtrInfo<decltype(result->host_ptr_info)>();
      result->client_request =
          input.TakeClientRequest<decltype(result->client_request)>();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::content::mojom::ServiceWorkerProviderInfoForStartWorker::DataView, ::content::mojom::ServiceWorkerProviderInfoForStartWorkerPtr>::Read(
    ::content::mojom::ServiceWorkerProviderInfoForStartWorker::DataView input,
    ::content::mojom::ServiceWorkerProviderInfoForStartWorkerPtr* output) {
  bool success = true;
  ::content::mojom::ServiceWorkerProviderInfoForStartWorkerPtr result(::content::mojom::ServiceWorkerProviderInfoForStartWorker::New());
  
      result->provider_id = input.provider_id();
      result->host_ptr_info =
          input.TakeHostPtrInfo<decltype(result->host_ptr_info)>();
      result->client_request =
          input.TakeClientRequest<decltype(result->client_request)>();
      result->script_loader_factory_ptr_info =
          input.TakeScriptLoaderFactoryPtrInfo<decltype(result->script_loader_factory_ptr_info)>();
      result->cache_storage =
          input.TakeCacheStorage<decltype(result->cache_storage)>();
      result->interface_provider =
          input.TakeInterfaceProvider<decltype(result->interface_provider)>();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::content::mojom::ServiceWorkerProviderHostInfo::DataView, ::content::mojom::ServiceWorkerProviderHostInfoPtr>::Read(
    ::content::mojom::ServiceWorkerProviderHostInfo::DataView input,
    ::content::mojom::ServiceWorkerProviderHostInfoPtr* output) {
  bool success = true;
  ::content::mojom::ServiceWorkerProviderHostInfoPtr result(::content::mojom::ServiceWorkerProviderHostInfo::New());
  
      result->provider_id = input.provider_id();
      result->route_id = input.route_id();
      if (!input.ReadType(&result->type))
        success = false;
      result->is_parent_frame_secure = input.is_parent_frame_secure();
      result->host_request =
          input.TakeHostRequest<decltype(result->host_request)>();
      result->client_ptr_info =
          input.TakeClientPtrInfo<decltype(result->client_ptr_info)>();
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif