// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_CONTENT_PUBLIC_MOJOM_NAVIGABLE_CONTENTS_MOJOM_H_
#define SERVICES_CONTENT_PUBLIC_MOJOM_NAVIGABLE_CONTENTS_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/content/public/mojom/navigable_contents.mojom-shared.h"
#include "mojo/public/mojom/base/unguessable_token.mojom.h"
#include "url/mojom/url.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace content {
namespace mojom {
class NavigableContents;
using NavigableContentsPtr = mojo::InterfacePtr<NavigableContents>;
using NavigableContentsPtrInfo = mojo::InterfacePtrInfo<NavigableContents>;
using ThreadSafeNavigableContentsPtr =
    mojo::ThreadSafeInterfacePtr<NavigableContents>;
using NavigableContentsRequest = mojo::InterfaceRequest<NavigableContents>;
using NavigableContentsAssociatedPtr =
    mojo::AssociatedInterfacePtr<NavigableContents>;
using ThreadSafeNavigableContentsAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<NavigableContents>;
using NavigableContentsAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<NavigableContents>;
using NavigableContentsAssociatedRequest =
    mojo::AssociatedInterfaceRequest<NavigableContents>;

class NavigableContentsClient;
using NavigableContentsClientPtr = mojo::InterfacePtr<NavigableContentsClient>;
using NavigableContentsClientPtrInfo = mojo::InterfacePtrInfo<NavigableContentsClient>;
using ThreadSafeNavigableContentsClientPtr =
    mojo::ThreadSafeInterfacePtr<NavigableContentsClient>;
using NavigableContentsClientRequest = mojo::InterfaceRequest<NavigableContentsClient>;
using NavigableContentsClientAssociatedPtr =
    mojo::AssociatedInterfacePtr<NavigableContentsClient>;
using ThreadSafeNavigableContentsClientAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<NavigableContentsClient>;
using NavigableContentsClientAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<NavigableContentsClient>;
using NavigableContentsClientAssociatedRequest =
    mojo::AssociatedInterfaceRequest<NavigableContentsClient>;


class NavigableContentsProxy;

template <typename ImplRefTraits>
class NavigableContentsStub;

class NavigableContentsRequestValidator;
class NavigableContentsResponseValidator;

class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContents
    : public NavigableContentsInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = NavigableContentsProxy;

  template <typename ImplRefTraits>
  using Stub_ = NavigableContentsStub<ImplRefTraits>;

  using RequestValidator_ = NavigableContentsRequestValidator;
  using ResponseValidator_ = NavigableContentsResponseValidator;
  enum MethodMinVersions : uint32_t {
    kNavigateMinVersion = 0,
    kCreateViewMinVersion = 0,
  };
  virtual ~NavigableContents() {}

  virtual void Navigate(const GURL& url) = 0;


  using CreateViewCallback = base::OnceCallback<void(const base::UnguessableToken&)>;
  virtual void CreateView(CreateViewCallback callback) = 0;
};
class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsInterceptorForTesting : public NavigableContents {
  virtual NavigableContents* GetForwardingInterface() = 0;
  void Navigate(const GURL& url) override;
  void CreateView(CreateViewCallback callback) override;
};
class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsAsyncWaiter {
 public:
  explicit NavigableContentsAsyncWaiter(NavigableContents* proxy);
  ~NavigableContentsAsyncWaiter();
  void CreateView(
      base::UnguessableToken* out_embed_token);

 private:
  NavigableContents* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(NavigableContentsAsyncWaiter);
};

class NavigableContentsClientProxy;

template <typename ImplRefTraits>
class NavigableContentsClientStub;

class NavigableContentsClientRequestValidator;

class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsClient
    : public NavigableContentsClientInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = NavigableContentsClientProxy;

  template <typename ImplRefTraits>
  using Stub_ = NavigableContentsClientStub<ImplRefTraits>;

  using RequestValidator_ = NavigableContentsClientRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kDidStopLoadingMinVersion = 0,
  };
  virtual ~NavigableContentsClient() {}

  virtual void DidStopLoading() = 0;
};
class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsClientInterceptorForTesting : public NavigableContentsClient {
  virtual NavigableContentsClient* GetForwardingInterface() = 0;
  void DidStopLoading() override;
};
class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsClientAsyncWaiter {
 public:
  explicit NavigableContentsClientAsyncWaiter(NavigableContentsClient* proxy);
  ~NavigableContentsClientAsyncWaiter();

 private:
  NavigableContentsClient* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(NavigableContentsClientAsyncWaiter);
};

class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsProxy
    : public NavigableContents {
 public:
  explicit NavigableContentsProxy(mojo::MessageReceiverWithResponder* receiver);
  void Navigate(const GURL& url) final;
  void CreateView(CreateViewCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsClientProxy
    : public NavigableContentsClient {
 public:
  explicit NavigableContentsClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void DidStopLoading() final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsStubDispatch {
 public:
  static bool Accept(NavigableContents* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      NavigableContents* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<NavigableContents>>
class NavigableContentsStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  NavigableContentsStub() {}
  ~NavigableContentsStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NavigableContentsStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NavigableContentsStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsClientStubDispatch {
 public:
  static bool Accept(NavigableContentsClient* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      NavigableContentsClient* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<NavigableContentsClient>>
class NavigableContentsClientStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  NavigableContentsClientStub() {}
  ~NavigableContentsClientStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NavigableContentsClientStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NavigableContentsClientStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsClientRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace content

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_CONTENT_PUBLIC_MOJOM_NAVIGABLE_CONTENTS_MOJOM_H_