// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_NETWORK_PUBLIC_MOJOM_PROXY_RESOLVING_SOCKET_MOJOM_H_
#define SERVICES_NETWORK_PUBLIC_MOJOM_PROXY_RESOLVING_SOCKET_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/network/public/mojom/proxy_resolving_socket.mojom-shared.h"
#include "net/interfaces/ip_endpoint.mojom.h"
#include "services/network/public/mojom/mutable_network_traffic_annotation_tag.mojom.h"
#include "url/mojom/url.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace network {
namespace mojom {
class ProxyResolvingSocket;
using ProxyResolvingSocketPtr = mojo::InterfacePtr<ProxyResolvingSocket>;
using ProxyResolvingSocketPtrInfo = mojo::InterfacePtrInfo<ProxyResolvingSocket>;
using ThreadSafeProxyResolvingSocketPtr =
    mojo::ThreadSafeInterfacePtr<ProxyResolvingSocket>;
using ProxyResolvingSocketRequest = mojo::InterfaceRequest<ProxyResolvingSocket>;
using ProxyResolvingSocketAssociatedPtr =
    mojo::AssociatedInterfacePtr<ProxyResolvingSocket>;
using ThreadSafeProxyResolvingSocketAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<ProxyResolvingSocket>;
using ProxyResolvingSocketAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<ProxyResolvingSocket>;
using ProxyResolvingSocketAssociatedRequest =
    mojo::AssociatedInterfaceRequest<ProxyResolvingSocket>;

class ProxyResolvingSocketFactory;
using ProxyResolvingSocketFactoryPtr = mojo::InterfacePtr<ProxyResolvingSocketFactory>;
using ProxyResolvingSocketFactoryPtrInfo = mojo::InterfacePtrInfo<ProxyResolvingSocketFactory>;
using ThreadSafeProxyResolvingSocketFactoryPtr =
    mojo::ThreadSafeInterfacePtr<ProxyResolvingSocketFactory>;
using ProxyResolvingSocketFactoryRequest = mojo::InterfaceRequest<ProxyResolvingSocketFactory>;
using ProxyResolvingSocketFactoryAssociatedPtr =
    mojo::AssociatedInterfacePtr<ProxyResolvingSocketFactory>;
using ThreadSafeProxyResolvingSocketFactoryAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<ProxyResolvingSocketFactory>;
using ProxyResolvingSocketFactoryAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<ProxyResolvingSocketFactory>;
using ProxyResolvingSocketFactoryAssociatedRequest =
    mojo::AssociatedInterfaceRequest<ProxyResolvingSocketFactory>;


class ProxyResolvingSocketProxy;

template <typename ImplRefTraits>
class ProxyResolvingSocketStub;

class ProxyResolvingSocketRequestValidator;
class ProxyResolvingSocketResponseValidator;

class  ProxyResolvingSocket
    : public ProxyResolvingSocketInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = ProxyResolvingSocketProxy;

  template <typename ImplRefTraits>
  using Stub_ = ProxyResolvingSocketStub<ImplRefTraits>;

  using RequestValidator_ = ProxyResolvingSocketRequestValidator;
  using ResponseValidator_ = ProxyResolvingSocketResponseValidator;
  enum MethodMinVersions : uint32_t {
    kGetPeerAddressMinVersion = 0,
  };
  virtual ~ProxyResolvingSocket() {}


  using GetPeerAddressCallback = base::OnceCallback<void(int32_t, const base::Optional<net::IPEndPoint>&)>;
  virtual void GetPeerAddress(GetPeerAddressCallback callback) = 0;
};
class  ProxyResolvingSocketInterceptorForTesting : public ProxyResolvingSocket {
  virtual ProxyResolvingSocket* GetForwardingInterface() = 0;
  void GetPeerAddress(GetPeerAddressCallback callback) override;
};
class  ProxyResolvingSocketAsyncWaiter {
 public:
  explicit ProxyResolvingSocketAsyncWaiter(ProxyResolvingSocket* proxy);
  ~ProxyResolvingSocketAsyncWaiter();
  void GetPeerAddress(
      int32_t* out_net_error, base::Optional<net::IPEndPoint>* out_peer_addr);

 private:
  ProxyResolvingSocket* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(ProxyResolvingSocketAsyncWaiter);
};

class ProxyResolvingSocketFactoryProxy;

template <typename ImplRefTraits>
class ProxyResolvingSocketFactoryStub;

class ProxyResolvingSocketFactoryRequestValidator;
class ProxyResolvingSocketFactoryResponseValidator;

class  ProxyResolvingSocketFactory
    : public ProxyResolvingSocketFactoryInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = ProxyResolvingSocketFactoryProxy;

  template <typename ImplRefTraits>
  using Stub_ = ProxyResolvingSocketFactoryStub<ImplRefTraits>;

  using RequestValidator_ = ProxyResolvingSocketFactoryRequestValidator;
  using ResponseValidator_ = ProxyResolvingSocketFactoryResponseValidator;
  enum MethodMinVersions : uint32_t {
    kCreateProxyResolvingSocketMinVersion = 0,
  };
  virtual ~ProxyResolvingSocketFactory() {}


  using CreateProxyResolvingSocketCallback = base::OnceCallback<void(int32_t, const base::Optional<net::IPEndPoint>&, mojo::ScopedDataPipeConsumerHandle, mojo::ScopedDataPipeProducerHandle)>;
  virtual void CreateProxyResolvingSocket(const GURL& url, bool use_tls, const net::MutableNetworkTrafficAnnotationTag& traffic_annotation, ProxyResolvingSocketRequest socket, CreateProxyResolvingSocketCallback callback) = 0;
};
class  ProxyResolvingSocketFactoryInterceptorForTesting : public ProxyResolvingSocketFactory {
  virtual ProxyResolvingSocketFactory* GetForwardingInterface() = 0;
  void CreateProxyResolvingSocket(const GURL& url, bool use_tls, const net::MutableNetworkTrafficAnnotationTag& traffic_annotation, ProxyResolvingSocketRequest socket, CreateProxyResolvingSocketCallback callback) override;
};
class  ProxyResolvingSocketFactoryAsyncWaiter {
 public:
  explicit ProxyResolvingSocketFactoryAsyncWaiter(ProxyResolvingSocketFactory* proxy);
  ~ProxyResolvingSocketFactoryAsyncWaiter();
  void CreateProxyResolvingSocket(
      const GURL& url, bool use_tls, const net::MutableNetworkTrafficAnnotationTag& traffic_annotation, ProxyResolvingSocketRequest socket, int32_t* out_result, base::Optional<net::IPEndPoint>* out_local_addr, mojo::ScopedDataPipeConsumerHandle* out_receive_stream, mojo::ScopedDataPipeProducerHandle* out_send_stream);

 private:
  ProxyResolvingSocketFactory* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(ProxyResolvingSocketFactoryAsyncWaiter);
};

class  ProxyResolvingSocketProxy
    : public ProxyResolvingSocket {
 public:
  explicit ProxyResolvingSocketProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetPeerAddress(GetPeerAddressCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  ProxyResolvingSocketFactoryProxy
    : public ProxyResolvingSocketFactory {
 public:
  explicit ProxyResolvingSocketFactoryProxy(mojo::MessageReceiverWithResponder* receiver);
  void CreateProxyResolvingSocket(const GURL& url, bool use_tls, const net::MutableNetworkTrafficAnnotationTag& traffic_annotation, ProxyResolvingSocketRequest socket, CreateProxyResolvingSocketCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  ProxyResolvingSocketStubDispatch {
 public:
  static bool Accept(ProxyResolvingSocket* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ProxyResolvingSocket* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ProxyResolvingSocket>>
class ProxyResolvingSocketStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ProxyResolvingSocketStub() {}
  ~ProxyResolvingSocketStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ProxyResolvingSocketStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ProxyResolvingSocketStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  ProxyResolvingSocketFactoryStubDispatch {
 public:
  static bool Accept(ProxyResolvingSocketFactory* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ProxyResolvingSocketFactory* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ProxyResolvingSocketFactory>>
class ProxyResolvingSocketFactoryStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ProxyResolvingSocketFactoryStub() {}
  ~ProxyResolvingSocketFactoryStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ProxyResolvingSocketFactoryStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ProxyResolvingSocketFactoryStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  ProxyResolvingSocketRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  ProxyResolvingSocketFactoryRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  ProxyResolvingSocketResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  ProxyResolvingSocketFactoryResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace network

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_NETWORK_PUBLIC_MOJOM_PROXY_RESOLVING_SOCKET_MOJOM_H_