// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "services/resource_coordinator/public/mojom/page_signal.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "services/resource_coordinator/public/mojom/page_signal.mojom-shared-message-ids.h"
#include "mojo/public/cpp/base/time_mojom_traits.h"
#include "services/resource_coordinator/public/cpp/coordination_unit_mojom_traits.h"
namespace resource_coordinator {
namespace mojom {
PageNavigationIdentity::PageNavigationIdentity()
    : page_cu_id(),
      navigation_id(),
      url() {}

PageNavigationIdentity::PageNavigationIdentity(
    const ::resource_coordinator::CoordinationUnitID& page_cu_id_in,
    int64_t navigation_id_in,
    const std::string& url_in)
    : page_cu_id(std::move(page_cu_id_in)),
      navigation_id(std::move(navigation_id_in)),
      url(std::move(url_in)) {}

PageNavigationIdentity::~PageNavigationIdentity() = default;

bool PageNavigationIdentity::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char PageSignalReceiver::Name_[] = "resource_coordinator.mojom.PageSignalReceiver";

PageSignalReceiverProxy::PageSignalReceiverProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void PageSignalReceiverProxy::NotifyPageAlmostIdle(
    const ::resource_coordinator::PageNavigationIdentity& in_page_navigation_id) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "resource_coordinator::mojom::PageSignalReceiver::NotifyPageAlmostIdle");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPageSignalReceiver_NotifyPageAlmostIdle_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::resource_coordinator::mojom::internal::PageSignalReceiver_NotifyPageAlmostIdle_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->page_navigation_id)::BaseType::BufferWriter
      page_navigation_id_writer;
  mojo::internal::Serialize<::resource_coordinator::mojom::PageNavigationIdentityDataView>(
      in_page_navigation_id, buffer, &page_navigation_id_writer, &serialization_context);
  params->page_navigation_id.Set(
      page_navigation_id_writer.is_null() ? nullptr : page_navigation_id_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->page_navigation_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null page_navigation_id in PageSignalReceiver.NotifyPageAlmostIdle request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(PageSignalReceiver::Name_);
  message.set_method_name("NotifyPageAlmostIdle");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PageSignalReceiverProxy::NotifyRendererIsBloated(
    const ::resource_coordinator::PageNavigationIdentity& in_page_navigation_id) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "resource_coordinator::mojom::PageSignalReceiver::NotifyRendererIsBloated");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPageSignalReceiver_NotifyRendererIsBloated_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::resource_coordinator::mojom::internal::PageSignalReceiver_NotifyRendererIsBloated_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->page_navigation_id)::BaseType::BufferWriter
      page_navigation_id_writer;
  mojo::internal::Serialize<::resource_coordinator::mojom::PageNavigationIdentityDataView>(
      in_page_navigation_id, buffer, &page_navigation_id_writer, &serialization_context);
  params->page_navigation_id.Set(
      page_navigation_id_writer.is_null() ? nullptr : page_navigation_id_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->page_navigation_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null page_navigation_id in PageSignalReceiver.NotifyRendererIsBloated request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(PageSignalReceiver::Name_);
  message.set_method_name("NotifyRendererIsBloated");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PageSignalReceiverProxy::SetExpectedTaskQueueingDuration(
    const ::resource_coordinator::PageNavigationIdentity& in_page_navigation_id, base::TimeDelta in_duration) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "resource_coordinator::mojom::PageSignalReceiver::SetExpectedTaskQueueingDuration");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPageSignalReceiver_SetExpectedTaskQueueingDuration_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::resource_coordinator::mojom::internal::PageSignalReceiver_SetExpectedTaskQueueingDuration_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->page_navigation_id)::BaseType::BufferWriter
      page_navigation_id_writer;
  mojo::internal::Serialize<::resource_coordinator::mojom::PageNavigationIdentityDataView>(
      in_page_navigation_id, buffer, &page_navigation_id_writer, &serialization_context);
  params->page_navigation_id.Set(
      page_navigation_id_writer.is_null() ? nullptr : page_navigation_id_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->page_navigation_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null page_navigation_id in PageSignalReceiver.SetExpectedTaskQueueingDuration request");
  typename decltype(params->duration)::BaseType::BufferWriter
      duration_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDeltaDataView>(
      in_duration, buffer, &duration_writer, &serialization_context);
  params->duration.Set(
      duration_writer.is_null() ? nullptr : duration_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->duration.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null duration in PageSignalReceiver.SetExpectedTaskQueueingDuration request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(PageSignalReceiver::Name_);
  message.set_method_name("SetExpectedTaskQueueingDuration");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PageSignalReceiverProxy::SetLifecycleState(
    const ::resource_coordinator::PageNavigationIdentity& in_page_navigation_id, ::resource_coordinator::mojom::LifecycleState in_state) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "resource_coordinator::mojom::PageSignalReceiver::SetLifecycleState");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPageSignalReceiver_SetLifecycleState_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::resource_coordinator::mojom::internal::PageSignalReceiver_SetLifecycleState_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->page_navigation_id)::BaseType::BufferWriter
      page_navigation_id_writer;
  mojo::internal::Serialize<::resource_coordinator::mojom::PageNavigationIdentityDataView>(
      in_page_navigation_id, buffer, &page_navigation_id_writer, &serialization_context);
  params->page_navigation_id.Set(
      page_navigation_id_writer.is_null() ? nullptr : page_navigation_id_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->page_navigation_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null page_navigation_id in PageSignalReceiver.SetLifecycleState request");
  mojo::internal::Serialize<::resource_coordinator::mojom::LifecycleState>(
      in_state, &params->state);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(PageSignalReceiver::Name_);
  message.set_method_name("SetLifecycleState");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PageSignalReceiverProxy::NotifyNonPersistentNotificationCreated(
    const ::resource_coordinator::PageNavigationIdentity& in_page_navigation_id) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "resource_coordinator::mojom::PageSignalReceiver::NotifyNonPersistentNotificationCreated");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPageSignalReceiver_NotifyNonPersistentNotificationCreated_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::resource_coordinator::mojom::internal::PageSignalReceiver_NotifyNonPersistentNotificationCreated_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->page_navigation_id)::BaseType::BufferWriter
      page_navigation_id_writer;
  mojo::internal::Serialize<::resource_coordinator::mojom::PageNavigationIdentityDataView>(
      in_page_navigation_id, buffer, &page_navigation_id_writer, &serialization_context);
  params->page_navigation_id.Set(
      page_navigation_id_writer.is_null() ? nullptr : page_navigation_id_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->page_navigation_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null page_navigation_id in PageSignalReceiver.NotifyNonPersistentNotificationCreated request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(PageSignalReceiver::Name_);
  message.set_method_name("NotifyNonPersistentNotificationCreated");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PageSignalReceiverProxy::OnLoadTimePerformanceEstimate(
    const ::resource_coordinator::PageNavigationIdentity& in_page_navigation_id, base::TimeDelta in_cpu_usage_estimate, uint64_t in_private_footprint_kb_estimate) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "resource_coordinator::mojom::PageSignalReceiver::OnLoadTimePerformanceEstimate");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPageSignalReceiver_OnLoadTimePerformanceEstimate_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::resource_coordinator::mojom::internal::PageSignalReceiver_OnLoadTimePerformanceEstimate_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->page_navigation_id)::BaseType::BufferWriter
      page_navigation_id_writer;
  mojo::internal::Serialize<::resource_coordinator::mojom::PageNavigationIdentityDataView>(
      in_page_navigation_id, buffer, &page_navigation_id_writer, &serialization_context);
  params->page_navigation_id.Set(
      page_navigation_id_writer.is_null() ? nullptr : page_navigation_id_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->page_navigation_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null page_navigation_id in PageSignalReceiver.OnLoadTimePerformanceEstimate request");
  typename decltype(params->cpu_usage_estimate)::BaseType::BufferWriter
      cpu_usage_estimate_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDeltaDataView>(
      in_cpu_usage_estimate, buffer, &cpu_usage_estimate_writer, &serialization_context);
  params->cpu_usage_estimate.Set(
      cpu_usage_estimate_writer.is_null() ? nullptr : cpu_usage_estimate_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->cpu_usage_estimate.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null cpu_usage_estimate in PageSignalReceiver.OnLoadTimePerformanceEstimate request");
  params->private_footprint_kb_estimate = in_private_footprint_kb_estimate;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(PageSignalReceiver::Name_);
  message.set_method_name("OnLoadTimePerformanceEstimate");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool PageSignalReceiverStubDispatch::Accept(
    PageSignalReceiver* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kPageSignalReceiver_NotifyPageAlmostIdle_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)resource_coordinator::mojom::PageSignalReceiver::NotifyPageAlmostIdle",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PageSignalReceiver_NotifyPageAlmostIdle_Params_Data* params =
          reinterpret_cast<internal::PageSignalReceiver_NotifyPageAlmostIdle_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::resource_coordinator::PageNavigationIdentity p_page_navigation_id{};
      PageSignalReceiver_NotifyPageAlmostIdle_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPageNavigationId(&p_page_navigation_id))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PageSignalReceiver::NotifyPageAlmostIdle deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->NotifyPageAlmostIdle(
std::move(p_page_navigation_id));
      return true;
    }
    case internal::kPageSignalReceiver_NotifyRendererIsBloated_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)resource_coordinator::mojom::PageSignalReceiver::NotifyRendererIsBloated",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PageSignalReceiver_NotifyRendererIsBloated_Params_Data* params =
          reinterpret_cast<internal::PageSignalReceiver_NotifyRendererIsBloated_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::resource_coordinator::PageNavigationIdentity p_page_navigation_id{};
      PageSignalReceiver_NotifyRendererIsBloated_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPageNavigationId(&p_page_navigation_id))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PageSignalReceiver::NotifyRendererIsBloated deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->NotifyRendererIsBloated(
std::move(p_page_navigation_id));
      return true;
    }
    case internal::kPageSignalReceiver_SetExpectedTaskQueueingDuration_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)resource_coordinator::mojom::PageSignalReceiver::SetExpectedTaskQueueingDuration",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PageSignalReceiver_SetExpectedTaskQueueingDuration_Params_Data* params =
          reinterpret_cast<internal::PageSignalReceiver_SetExpectedTaskQueueingDuration_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::resource_coordinator::PageNavigationIdentity p_page_navigation_id{};
      base::TimeDelta p_duration{};
      PageSignalReceiver_SetExpectedTaskQueueingDuration_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPageNavigationId(&p_page_navigation_id))
        success = false;
      if (!input_data_view.ReadDuration(&p_duration))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PageSignalReceiver::SetExpectedTaskQueueingDuration deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetExpectedTaskQueueingDuration(
std::move(p_page_navigation_id), 
std::move(p_duration));
      return true;
    }
    case internal::kPageSignalReceiver_SetLifecycleState_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)resource_coordinator::mojom::PageSignalReceiver::SetLifecycleState",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PageSignalReceiver_SetLifecycleState_Params_Data* params =
          reinterpret_cast<internal::PageSignalReceiver_SetLifecycleState_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::resource_coordinator::PageNavigationIdentity p_page_navigation_id{};
      ::resource_coordinator::mojom::LifecycleState p_state{};
      PageSignalReceiver_SetLifecycleState_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPageNavigationId(&p_page_navigation_id))
        success = false;
      if (!input_data_view.ReadState(&p_state))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PageSignalReceiver::SetLifecycleState deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetLifecycleState(
std::move(p_page_navigation_id), 
std::move(p_state));
      return true;
    }
    case internal::kPageSignalReceiver_NotifyNonPersistentNotificationCreated_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)resource_coordinator::mojom::PageSignalReceiver::NotifyNonPersistentNotificationCreated",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PageSignalReceiver_NotifyNonPersistentNotificationCreated_Params_Data* params =
          reinterpret_cast<internal::PageSignalReceiver_NotifyNonPersistentNotificationCreated_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::resource_coordinator::PageNavigationIdentity p_page_navigation_id{};
      PageSignalReceiver_NotifyNonPersistentNotificationCreated_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPageNavigationId(&p_page_navigation_id))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PageSignalReceiver::NotifyNonPersistentNotificationCreated deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->NotifyNonPersistentNotificationCreated(
std::move(p_page_navigation_id));
      return true;
    }
    case internal::kPageSignalReceiver_OnLoadTimePerformanceEstimate_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)resource_coordinator::mojom::PageSignalReceiver::OnLoadTimePerformanceEstimate",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PageSignalReceiver_OnLoadTimePerformanceEstimate_Params_Data* params =
          reinterpret_cast<internal::PageSignalReceiver_OnLoadTimePerformanceEstimate_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::resource_coordinator::PageNavigationIdentity p_page_navigation_id{};
      base::TimeDelta p_cpu_usage_estimate{};
      uint64_t p_private_footprint_kb_estimate{};
      PageSignalReceiver_OnLoadTimePerformanceEstimate_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPageNavigationId(&p_page_navigation_id))
        success = false;
      if (!input_data_view.ReadCpuUsageEstimate(&p_cpu_usage_estimate))
        success = false;
      p_private_footprint_kb_estimate = input_data_view.private_footprint_kb_estimate();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PageSignalReceiver::OnLoadTimePerformanceEstimate deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnLoadTimePerformanceEstimate(
std::move(p_page_navigation_id), 
std::move(p_cpu_usage_estimate), 
std::move(p_private_footprint_kb_estimate));
      return true;
    }
  }
  return false;
}

// static
bool PageSignalReceiverStubDispatch::AcceptWithResponder(
    PageSignalReceiver* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kPageSignalReceiver_NotifyPageAlmostIdle_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)resource_coordinator::mojom::PageSignalReceiver::NotifyPageAlmostIdle",
               "message", message->name());
#endif
      break;
    }
    case internal::kPageSignalReceiver_NotifyRendererIsBloated_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)resource_coordinator::mojom::PageSignalReceiver::NotifyRendererIsBloated",
               "message", message->name());
#endif
      break;
    }
    case internal::kPageSignalReceiver_SetExpectedTaskQueueingDuration_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)resource_coordinator::mojom::PageSignalReceiver::SetExpectedTaskQueueingDuration",
               "message", message->name());
#endif
      break;
    }
    case internal::kPageSignalReceiver_SetLifecycleState_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)resource_coordinator::mojom::PageSignalReceiver::SetLifecycleState",
               "message", message->name());
#endif
      break;
    }
    case internal::kPageSignalReceiver_NotifyNonPersistentNotificationCreated_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)resource_coordinator::mojom::PageSignalReceiver::NotifyNonPersistentNotificationCreated",
               "message", message->name());
#endif
      break;
    }
    case internal::kPageSignalReceiver_OnLoadTimePerformanceEstimate_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)resource_coordinator::mojom::PageSignalReceiver::OnLoadTimePerformanceEstimate",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool PageSignalReceiverRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "PageSignalReceiver RequestValidator");

  switch (message->header()->name) {
    case internal::kPageSignalReceiver_NotifyPageAlmostIdle_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PageSignalReceiver_NotifyPageAlmostIdle_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPageSignalReceiver_NotifyRendererIsBloated_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PageSignalReceiver_NotifyRendererIsBloated_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPageSignalReceiver_SetExpectedTaskQueueingDuration_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PageSignalReceiver_SetExpectedTaskQueueingDuration_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPageSignalReceiver_SetLifecycleState_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PageSignalReceiver_SetLifecycleState_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPageSignalReceiver_NotifyNonPersistentNotificationCreated_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PageSignalReceiver_NotifyNonPersistentNotificationCreated_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPageSignalReceiver_OnLoadTimePerformanceEstimate_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PageSignalReceiver_OnLoadTimePerformanceEstimate_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void PageSignalReceiverInterceptorForTesting::NotifyPageAlmostIdle(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id) {
  GetForwardingInterface()->NotifyPageAlmostIdle(std::move(page_navigation_id));
}
void PageSignalReceiverInterceptorForTesting::NotifyRendererIsBloated(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id) {
  GetForwardingInterface()->NotifyRendererIsBloated(std::move(page_navigation_id));
}
void PageSignalReceiverInterceptorForTesting::SetExpectedTaskQueueingDuration(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id, base::TimeDelta duration) {
  GetForwardingInterface()->SetExpectedTaskQueueingDuration(std::move(page_navigation_id), std::move(duration));
}
void PageSignalReceiverInterceptorForTesting::SetLifecycleState(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id, ::resource_coordinator::mojom::LifecycleState state) {
  GetForwardingInterface()->SetLifecycleState(std::move(page_navigation_id), std::move(state));
}
void PageSignalReceiverInterceptorForTesting::NotifyNonPersistentNotificationCreated(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id) {
  GetForwardingInterface()->NotifyNonPersistentNotificationCreated(std::move(page_navigation_id));
}
void PageSignalReceiverInterceptorForTesting::OnLoadTimePerformanceEstimate(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id, base::TimeDelta cpu_usage_estimate, uint64_t private_footprint_kb_estimate) {
  GetForwardingInterface()->OnLoadTimePerformanceEstimate(std::move(page_navigation_id), std::move(cpu_usage_estimate), std::move(private_footprint_kb_estimate));
}
PageSignalReceiverAsyncWaiter::PageSignalReceiverAsyncWaiter(
    PageSignalReceiver* proxy) : proxy_(proxy) {}

PageSignalReceiverAsyncWaiter::~PageSignalReceiverAsyncWaiter() = default;


const char PageSignalGenerator::Name_[] = "resource_coordinator.mojom.PageSignalGenerator";

PageSignalGeneratorProxy::PageSignalGeneratorProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void PageSignalGeneratorProxy::AddReceiver(
    PageSignalReceiverPtr in_receiver) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "resource_coordinator::mojom::PageSignalGenerator::AddReceiver");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kPageSignalGenerator_AddReceiver_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::resource_coordinator::mojom::internal::PageSignalGenerator_AddReceiver_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::resource_coordinator::mojom::PageSignalReceiverPtrDataView>(
      in_receiver, &params->receiver, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->receiver),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid receiver in PageSignalGenerator.AddReceiver request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(PageSignalGenerator::Name_);
  message.set_method_name("AddReceiver");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool PageSignalGeneratorStubDispatch::Accept(
    PageSignalGenerator* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kPageSignalGenerator_AddReceiver_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)resource_coordinator::mojom::PageSignalGenerator::AddReceiver",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PageSignalGenerator_AddReceiver_Params_Data* params =
          reinterpret_cast<internal::PageSignalGenerator_AddReceiver_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      PageSignalReceiverPtr p_receiver{};
      PageSignalGenerator_AddReceiver_ParamsDataView input_data_view(params, &serialization_context);
      
      p_receiver =
          input_data_view.TakeReceiver<decltype(p_receiver)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PageSignalGenerator::AddReceiver deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->AddReceiver(
std::move(p_receiver));
      return true;
    }
  }
  return false;
}

// static
bool PageSignalGeneratorStubDispatch::AcceptWithResponder(
    PageSignalGenerator* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kPageSignalGenerator_AddReceiver_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)resource_coordinator::mojom::PageSignalGenerator::AddReceiver",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool PageSignalGeneratorRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "PageSignalGenerator RequestValidator");

  switch (message->header()->name) {
    case internal::kPageSignalGenerator_AddReceiver_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PageSignalGenerator_AddReceiver_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void PageSignalGeneratorInterceptorForTesting::AddReceiver(PageSignalReceiverPtr receiver) {
  GetForwardingInterface()->AddReceiver(std::move(receiver));
}
PageSignalGeneratorAsyncWaiter::PageSignalGeneratorAsyncWaiter(
    PageSignalGenerator* proxy) : proxy_(proxy) {}

PageSignalGeneratorAsyncWaiter::~PageSignalGeneratorAsyncWaiter() = default;


}  // namespace mojom
}  // namespace resource_coordinator

namespace mojo {


// static
bool StructTraits<::resource_coordinator::mojom::PageNavigationIdentity::DataView, ::resource_coordinator::mojom::PageNavigationIdentityPtr>::Read(
    ::resource_coordinator::mojom::PageNavigationIdentity::DataView input,
    ::resource_coordinator::mojom::PageNavigationIdentityPtr* output) {
  bool success = true;
  ::resource_coordinator::mojom::PageNavigationIdentityPtr result(::resource_coordinator::mojom::PageNavigationIdentity::New());
  
      if (!input.ReadPageCuId(&result->page_cu_id))
        success = false;
      result->navigation_id = input.navigation_id();
      if (!input.ReadUrl(&result->url))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif