// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_UI_PUBLIC_INTERFACES_WINDOW_MANAGER_CONSTANTS_MOJOM_SHARED_H_
#define SERVICES_UI_PUBLIC_INTERFACES_WINDOW_MANAGER_CONSTANTS_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/ui/public/interfaces/window_manager_constants.mojom-shared-internal.h"
#include "ui/display/mojo/display.mojom-shared.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace ui {
namespace mojom {
class WmViewportMetricsDataView;

class BlockingContainersDataView;



}  // namespace mojom
}  // namespace ui

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::ui::mojom::WmViewportMetricsDataView> {
  using Data = ::ui::mojom::internal::WmViewportMetrics_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::ui::mojom::BlockingContainersDataView> {
  using Data = ::ui::mojom::internal::BlockingContainers_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace ui {
namespace mojom {

enum class WindowManagerErrorCode : int32_t {
  SUCCESS,
  ACCESS_DENIED,
  kMinValue = 0,
  kMaxValue = 1,
};

inline std::ostream& operator<<(std::ostream& os, WindowManagerErrorCode value) {
  switch(value) {
    case WindowManagerErrorCode::SUCCESS:
      return os << "WindowManagerErrorCode::SUCCESS";
    case WindowManagerErrorCode::ACCESS_DENIED:
      return os << "WindowManagerErrorCode::ACCESS_DENIED";
    default:
      return os << "Unknown WindowManagerErrorCode value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(WindowManagerErrorCode value) {
  return internal::WindowManagerErrorCode_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
class WmViewportMetricsDataView {
 public:
  WmViewportMetricsDataView() {}

  WmViewportMetricsDataView(
      internal::WmViewportMetrics_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetBoundsInPixelsDataView(
      ::gfx::mojom::RectDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadBoundsInPixels(UserType* output) {
    auto* pointer = data_->bounds_in_pixels.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectDataView>(
        pointer, output, context_);
  }
  float device_scale_factor() const {
    return data_->device_scale_factor;
  }
  float ui_scale_factor() const {
    return data_->ui_scale_factor;
  }
 private:
  internal::WmViewportMetrics_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BlockingContainersDataView {
 public:
  BlockingContainersDataView() {}

  BlockingContainersDataView(
      internal::BlockingContainers_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  uint64_t system_modal_container_id() const {
    return data_->system_modal_container_id;
  }
  uint64_t min_container_id() const {
    return data_->min_container_id;
  }
 private:
  internal::BlockingContainers_Data* data_ = nullptr;
};



}  // namespace mojom
}  // namespace ui

namespace std {

template <>
struct hash<::ui::mojom::WindowManagerErrorCode>
    : public mojo::internal::EnumHashImpl<::ui::mojom::WindowManagerErrorCode> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::ui::mojom::WindowManagerErrorCode, ::ui::mojom::WindowManagerErrorCode> {
  static ::ui::mojom::WindowManagerErrorCode ToMojom(::ui::mojom::WindowManagerErrorCode input) { return input; }
  static bool FromMojom(::ui::mojom::WindowManagerErrorCode input, ::ui::mojom::WindowManagerErrorCode* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::WindowManagerErrorCode, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::ui::mojom::WindowManagerErrorCode, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::ui::mojom::WindowManagerErrorCode>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::WmViewportMetricsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::ui::mojom::WmViewportMetricsDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::ui::mojom::internal::WmViewportMetrics_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::bounds_in_pixels, input, custom_context)) in_bounds_in_pixels = CallWithContext(Traits::bounds_in_pixels, input, custom_context);
    typename decltype((*output)->bounds_in_pixels)::BaseType::BufferWriter
        bounds_in_pixels_writer;
    mojo::internal::Serialize<::gfx::mojom::RectDataView>(
        in_bounds_in_pixels, buffer, &bounds_in_pixels_writer, context);
    (*output)->bounds_in_pixels.Set(
        bounds_in_pixels_writer.is_null() ? nullptr : bounds_in_pixels_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->bounds_in_pixels.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null bounds_in_pixels in WmViewportMetrics struct");
    (*output)->device_scale_factor = CallWithContext(Traits::device_scale_factor, input, custom_context);
    (*output)->ui_scale_factor = CallWithContext(Traits::ui_scale_factor, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::ui::mojom::internal::WmViewportMetrics_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::ui::mojom::WmViewportMetricsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::BlockingContainersDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::ui::mojom::BlockingContainersDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::ui::mojom::internal::BlockingContainers_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    (*output)->system_modal_container_id = CallWithContext(Traits::system_modal_container_id, input, custom_context);
    (*output)->min_container_id = CallWithContext(Traits::min_container_id, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::ui::mojom::internal::BlockingContainers_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::ui::mojom::BlockingContainersDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace ui {
namespace mojom {

inline void WmViewportMetricsDataView::GetBoundsInPixelsDataView(
    ::gfx::mojom::RectDataView* output) {
  auto pointer = data_->bounds_in_pixels.Get();
  *output = ::gfx::mojom::RectDataView(pointer, context_);
}





}  // namespace mojom
}  // namespace ui

#endif  // SERVICES_UI_PUBLIC_INTERFACES_WINDOW_MANAGER_CONSTANTS_MOJOM_SHARED_H_
