// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_PUBLIC_MOJOM_LOADER_NAVIGATION_PREDICTOR_MOJOM_SHARED_H_
#define THIRD_PARTY_BLINK_PUBLIC_MOJOM_LOADER_NAVIGATION_PREDICTOR_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "third_party/blink/public/mojom/loader/navigation_predictor.mojom-shared-internal.h"
#include "url/mojom/url.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace blink {
namespace mojom {
class AnchorElementMetricsDataView;



}  // namespace mojom
}  // namespace blink

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::blink::mojom::AnchorElementMetricsDataView> {
  using Data = ::blink::mojom::internal::AnchorElementMetrics_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace blink {
namespace mojom {
// Interface base classes. They are used for type safety check.
class AnchorElementMetricsHostInterfaceBase {};

using AnchorElementMetricsHostPtrDataView =
    mojo::InterfacePtrDataView<AnchorElementMetricsHostInterfaceBase>;
using AnchorElementMetricsHostRequestDataView =
    mojo::InterfaceRequestDataView<AnchorElementMetricsHostInterfaceBase>;
using AnchorElementMetricsHostAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<AnchorElementMetricsHostInterfaceBase>;
using AnchorElementMetricsHostAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<AnchorElementMetricsHostInterfaceBase>;
class AnchorElementMetricsDataView {
 public:
  AnchorElementMetricsDataView() {}

  AnchorElementMetricsDataView(
      internal::AnchorElementMetrics_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  float ratio_area() const {
    return data_->ratio_area;
  }
  float ratio_distance_root_top() const {
    return data_->ratio_distance_root_top;
  }
  float ratio_distance_center_to_visible_top() const {
    return data_->ratio_distance_center_to_visible_top;
  }
  inline void GetTargetUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTargetUrl(UserType* output) {
    auto* pointer = data_->target_url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::AnchorElementMetrics_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AnchorElementMetricsHost_UpdateAnchorElementMetrics_ParamsDataView {
 public:
  AnchorElementMetricsHost_UpdateAnchorElementMetrics_ParamsDataView() {}

  AnchorElementMetricsHost_UpdateAnchorElementMetrics_ParamsDataView(
      internal::AnchorElementMetricsHost_UpdateAnchorElementMetrics_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetMetricsDataView(
      AnchorElementMetricsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMetrics(UserType* output) {
    auto* pointer = data_->metrics.Get();
    return mojo::internal::Deserialize<::blink::mojom::AnchorElementMetricsDataView>(
        pointer, output, context_);
  }
 private:
  internal::AnchorElementMetricsHost_UpdateAnchorElementMetrics_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace blink

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::AnchorElementMetricsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::blink::mojom::AnchorElementMetricsDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::blink::mojom::internal::AnchorElementMetrics_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    (*output)->ratio_area = CallWithContext(Traits::ratio_area, input, custom_context);
    (*output)->ratio_distance_root_top = CallWithContext(Traits::ratio_distance_root_top, input, custom_context);
    (*output)->ratio_distance_center_to_visible_top = CallWithContext(Traits::ratio_distance_center_to_visible_top, input, custom_context);
    decltype(CallWithContext(Traits::target_url, input, custom_context)) in_target_url = CallWithContext(Traits::target_url, input, custom_context);
    typename decltype((*output)->target_url)::BaseType::BufferWriter
        target_url_writer;
    mojo::internal::Serialize<::url::mojom::UrlDataView>(
        in_target_url, buffer, &target_url_writer, context);
    (*output)->target_url.Set(
        target_url_writer.is_null() ? nullptr : target_url_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->target_url.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null target_url in AnchorElementMetrics struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::blink::mojom::internal::AnchorElementMetrics_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::blink::mojom::AnchorElementMetricsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace blink {
namespace mojom {

inline void AnchorElementMetricsDataView::GetTargetUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->target_url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}


inline void AnchorElementMetricsHost_UpdateAnchorElementMetrics_ParamsDataView::GetMetricsDataView(
    AnchorElementMetricsDataView* output) {
  auto pointer = data_->metrics.Get();
  *output = AnchorElementMetricsDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace blink

#endif  // THIRD_PARTY_BLINK_PUBLIC_MOJOM_LOADER_NAVIGATION_PREDICTOR_MOJOM_SHARED_H_
