// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef UI_OZONE_PUBLIC_INTERFACES_OVERLAY_SURFACE_CANDIDATE_MOJOM_SHARED_H_
#define UI_OZONE_PUBLIC_INTERFACES_OVERLAY_SURFACE_CANDIDATE_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "ui/ozone/public/interfaces/overlay_surface_candidate.mojom-shared-internal.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"
#include "ui/gfx/mojo/buffer_types.mojom-shared.h"
#include "ui/gfx/mojo/overlay_transform.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace ui {
namespace ozone {
namespace mojom {
class OverlaySurfaceCandidateDataView;



}  // namespace mojom
}  // namespace ozone
}  // namespace ui

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::ui::ozone::mojom::OverlaySurfaceCandidateDataView> {
  using Data = ::ui::ozone::mojom::internal::OverlaySurfaceCandidate_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace ui {
namespace ozone {
namespace mojom {

enum class OverlayStatus : int32_t {
  OVERLAY_STATUS_PENDING,
  OVERLAY_STATUS_ABLE,
  OVERLAY_STATUS_NOT,
  kMinValue = 0,
  kMaxValue = 2,
};

inline std::ostream& operator<<(std::ostream& os, OverlayStatus value) {
  switch(value) {
    case OverlayStatus::OVERLAY_STATUS_PENDING:
      return os << "OverlayStatus::OVERLAY_STATUS_PENDING";
    case OverlayStatus::OVERLAY_STATUS_ABLE:
      return os << "OverlayStatus::OVERLAY_STATUS_ABLE";
    case OverlayStatus::OVERLAY_STATUS_NOT:
      return os << "OverlayStatus::OVERLAY_STATUS_NOT";
    default:
      return os << "Unknown OverlayStatus value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(OverlayStatus value) {
  return internal::OverlayStatus_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
class OverlaySurfaceCandidateDataView {
 public:
  OverlaySurfaceCandidateDataView() {}

  OverlaySurfaceCandidateDataView(
      internal::OverlaySurfaceCandidate_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTransform(UserType* output) const {
    auto data_value = data_->transform;
    return mojo::internal::Deserialize<::gfx::mojom::OverlayTransform>(
        data_value, output);
  }

  ::gfx::mojom::OverlayTransform transform() const {
    return static_cast<::gfx::mojom::OverlayTransform>(data_->transform);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFormat(UserType* output) const {
    auto data_value = data_->format;
    return mojo::internal::Deserialize<::gfx::mojom::BufferFormat>(
        data_value, output);
  }

  ::gfx::mojom::BufferFormat format() const {
    return static_cast<::gfx::mojom::BufferFormat>(data_->format);
  }
  inline void GetBufferSizeDataView(
      ::gfx::mojom::SizeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadBufferSize(UserType* output) {
    auto* pointer = data_->buffer_size.Get();
    return mojo::internal::Deserialize<::gfx::mojom::SizeDataView>(
        pointer, output, context_);
  }
  inline void GetDisplayRectDataView(
      ::gfx::mojom::RectFDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDisplayRect(UserType* output) {
    auto* pointer = data_->display_rect.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectFDataView>(
        pointer, output, context_);
  }
  inline void GetCropRectDataView(
      ::gfx::mojom::RectFDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCropRect(UserType* output) {
    auto* pointer = data_->crop_rect.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectFDataView>(
        pointer, output, context_);
  }
  inline void GetClipRectDataView(
      ::gfx::mojom::RectDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadClipRect(UserType* output) {
    auto* pointer = data_->clip_rect.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectDataView>(
        pointer, output, context_);
  }
  bool is_clipped() const {
    return data_->is_clipped;
  }
  int32_t plane_z_order() const {
    return data_->plane_z_order;
  }
  bool overlay_handled() const {
    return data_->overlay_handled;
  }
 private:
  internal::OverlaySurfaceCandidate_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace ozone
}  // namespace ui

namespace std {

template <>
struct hash<::ui::ozone::mojom::OverlayStatus>
    : public mojo::internal::EnumHashImpl<::ui::ozone::mojom::OverlayStatus> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::ui::ozone::mojom::OverlayStatus, ::ui::ozone::mojom::OverlayStatus> {
  static ::ui::ozone::mojom::OverlayStatus ToMojom(::ui::ozone::mojom::OverlayStatus input) { return input; }
  static bool FromMojom(::ui::ozone::mojom::OverlayStatus input, ::ui::ozone::mojom::OverlayStatus* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::ozone::mojom::OverlayStatus, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::ui::ozone::mojom::OverlayStatus, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::ui::ozone::mojom::OverlayStatus>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::ozone::mojom::OverlaySurfaceCandidateDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::ui::ozone::mojom::OverlaySurfaceCandidateDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::ui::ozone::mojom::internal::OverlaySurfaceCandidate_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    mojo::internal::Serialize<::gfx::mojom::OverlayTransform>(
        CallWithContext(Traits::transform, input, custom_context), &(*output)->transform);
    mojo::internal::Serialize<::gfx::mojom::BufferFormat>(
        CallWithContext(Traits::format, input, custom_context), &(*output)->format);
    decltype(CallWithContext(Traits::buffer_size, input, custom_context)) in_buffer_size = CallWithContext(Traits::buffer_size, input, custom_context);
    typename decltype((*output)->buffer_size)::BaseType::BufferWriter
        buffer_size_writer;
    mojo::internal::Serialize<::gfx::mojom::SizeDataView>(
        in_buffer_size, buffer, &buffer_size_writer, context);
    (*output)->buffer_size.Set(
        buffer_size_writer.is_null() ? nullptr : buffer_size_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->buffer_size.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null buffer_size in OverlaySurfaceCandidate struct");
    decltype(CallWithContext(Traits::display_rect, input, custom_context)) in_display_rect = CallWithContext(Traits::display_rect, input, custom_context);
    typename decltype((*output)->display_rect)::BaseType::BufferWriter
        display_rect_writer;
    mojo::internal::Serialize<::gfx::mojom::RectFDataView>(
        in_display_rect, buffer, &display_rect_writer, context);
    (*output)->display_rect.Set(
        display_rect_writer.is_null() ? nullptr : display_rect_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->display_rect.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null display_rect in OverlaySurfaceCandidate struct");
    decltype(CallWithContext(Traits::crop_rect, input, custom_context)) in_crop_rect = CallWithContext(Traits::crop_rect, input, custom_context);
    typename decltype((*output)->crop_rect)::BaseType::BufferWriter
        crop_rect_writer;
    mojo::internal::Serialize<::gfx::mojom::RectFDataView>(
        in_crop_rect, buffer, &crop_rect_writer, context);
    (*output)->crop_rect.Set(
        crop_rect_writer.is_null() ? nullptr : crop_rect_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->crop_rect.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null crop_rect in OverlaySurfaceCandidate struct");
    decltype(CallWithContext(Traits::clip_rect, input, custom_context)) in_clip_rect = CallWithContext(Traits::clip_rect, input, custom_context);
    typename decltype((*output)->clip_rect)::BaseType::BufferWriter
        clip_rect_writer;
    mojo::internal::Serialize<::gfx::mojom::RectDataView>(
        in_clip_rect, buffer, &clip_rect_writer, context);
    (*output)->clip_rect.Set(
        clip_rect_writer.is_null() ? nullptr : clip_rect_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->clip_rect.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null clip_rect in OverlaySurfaceCandidate struct");
    (*output)->is_clipped = CallWithContext(Traits::is_clipped, input, custom_context);
    (*output)->plane_z_order = CallWithContext(Traits::plane_z_order, input, custom_context);
    (*output)->overlay_handled = CallWithContext(Traits::overlay_handled, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::ui::ozone::mojom::internal::OverlaySurfaceCandidate_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::ui::ozone::mojom::OverlaySurfaceCandidateDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace ui {
namespace ozone {
namespace mojom {

inline void OverlaySurfaceCandidateDataView::GetBufferSizeDataView(
    ::gfx::mojom::SizeDataView* output) {
  auto pointer = data_->buffer_size.Get();
  *output = ::gfx::mojom::SizeDataView(pointer, context_);
}
inline void OverlaySurfaceCandidateDataView::GetDisplayRectDataView(
    ::gfx::mojom::RectFDataView* output) {
  auto pointer = data_->display_rect.Get();
  *output = ::gfx::mojom::RectFDataView(pointer, context_);
}
inline void OverlaySurfaceCandidateDataView::GetCropRectDataView(
    ::gfx::mojom::RectFDataView* output) {
  auto pointer = data_->crop_rect.Get();
  *output = ::gfx::mojom::RectFDataView(pointer, context_);
}
inline void OverlaySurfaceCandidateDataView::GetClipRectDataView(
    ::gfx::mojom::RectDataView* output) {
  auto pointer = data_->clip_rect.Get();
  *output = ::gfx::mojom::RectDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace ozone
}  // namespace ui

#endif  // UI_OZONE_PUBLIC_INTERFACES_OVERLAY_SURFACE_CANDIDATE_MOJOM_SHARED_H_
