// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef UI_OZONE_PUBLIC_INTERFACES_OVERLAY_SURFACE_CANDIDATE_MOJOM_H_
#define UI_OZONE_PUBLIC_INTERFACES_OVERLAY_SURFACE_CANDIDATE_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "ui/ozone/public/interfaces/overlay_surface_candidate.mojom-shared.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include "ui/gfx/mojo/buffer_types.mojom.h"
#include "ui/gfx/mojo/overlay_transform.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "ui/ozone/public/overlay_surface_candidate.h"


namespace ui {
namespace ozone {
namespace mojom {
class OverlaySurfaceCandidate;
using OverlaySurfaceCandidatePtr = mojo::StructPtr<OverlaySurfaceCandidate>;






class  OverlaySurfaceCandidate {
 public:
  using DataView = OverlaySurfaceCandidateDataView;
  using Data_ = internal::OverlaySurfaceCandidate_Data;

  template <typename... Args>
  static OverlaySurfaceCandidatePtr New(Args&&... args) {
    return OverlaySurfaceCandidatePtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static OverlaySurfaceCandidatePtr From(const U& u) {
    return mojo::TypeConverter<OverlaySurfaceCandidatePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, OverlaySurfaceCandidate>::Convert(*this);
  }


  OverlaySurfaceCandidate();

  OverlaySurfaceCandidate(
      gfx::OverlayTransform transform,
      gfx::BufferFormat format,
      const gfx::Size& buffer_size,
      const gfx::RectF& display_rect,
      const gfx::RectF& crop_rect,
      const gfx::Rect& clip_rect,
      bool is_clipped,
      int32_t plane_z_order,
      bool overlay_handled);

  ~OverlaySurfaceCandidate();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = OverlaySurfaceCandidatePtr>
  OverlaySurfaceCandidatePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, OverlaySurfaceCandidate>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        OverlaySurfaceCandidate::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        OverlaySurfaceCandidate::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::OverlaySurfaceCandidate_UnserializedMessageContext<
            UserType, OverlaySurfaceCandidate::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<OverlaySurfaceCandidate::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return OverlaySurfaceCandidate::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::OverlaySurfaceCandidate_UnserializedMessageContext<
            UserType, OverlaySurfaceCandidate::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<OverlaySurfaceCandidate::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  gfx::OverlayTransform transform;
  gfx::BufferFormat format;
  gfx::Size buffer_size;
  gfx::RectF display_rect;
  gfx::RectF crop_rect;
  gfx::Rect clip_rect;
  bool is_clipped;
  int32_t plane_z_order;
  bool overlay_handled;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

template <typename StructPtrType>
OverlaySurfaceCandidatePtr OverlaySurfaceCandidate::Clone() const {
  return New(
      mojo::Clone(transform),
      mojo::Clone(format),
      mojo::Clone(buffer_size),
      mojo::Clone(display_rect),
      mojo::Clone(crop_rect),
      mojo::Clone(clip_rect),
      mojo::Clone(is_clipped),
      mojo::Clone(plane_z_order),
      mojo::Clone(overlay_handled)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, OverlaySurfaceCandidate>::value>::type*>
bool OverlaySurfaceCandidate::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->transform, other_struct.transform))
    return false;
  if (!mojo::Equals(this->format, other_struct.format))
    return false;
  if (!mojo::Equals(this->buffer_size, other_struct.buffer_size))
    return false;
  if (!mojo::Equals(this->display_rect, other_struct.display_rect))
    return false;
  if (!mojo::Equals(this->crop_rect, other_struct.crop_rect))
    return false;
  if (!mojo::Equals(this->clip_rect, other_struct.clip_rect))
    return false;
  if (!mojo::Equals(this->is_clipped, other_struct.is_clipped))
    return false;
  if (!mojo::Equals(this->plane_z_order, other_struct.plane_z_order))
    return false;
  if (!mojo::Equals(this->overlay_handled, other_struct.overlay_handled))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace ozone
}  // namespace ui

namespace mojo {


template <>
struct  StructTraits<::ui::ozone::mojom::OverlaySurfaceCandidate::DataView,
                                         ::ui::ozone::mojom::OverlaySurfaceCandidatePtr> {
  static bool IsNull(const ::ui::ozone::mojom::OverlaySurfaceCandidatePtr& input) { return !input; }
  static void SetToNull(::ui::ozone::mojom::OverlaySurfaceCandidatePtr* output) { output->reset(); }

  static decltype(::ui::ozone::mojom::OverlaySurfaceCandidate::transform) transform(
      const ::ui::ozone::mojom::OverlaySurfaceCandidatePtr& input) {
    return input->transform;
  }

  static decltype(::ui::ozone::mojom::OverlaySurfaceCandidate::format) format(
      const ::ui::ozone::mojom::OverlaySurfaceCandidatePtr& input) {
    return input->format;
  }

  static const decltype(::ui::ozone::mojom::OverlaySurfaceCandidate::buffer_size)& buffer_size(
      const ::ui::ozone::mojom::OverlaySurfaceCandidatePtr& input) {
    return input->buffer_size;
  }

  static const decltype(::ui::ozone::mojom::OverlaySurfaceCandidate::display_rect)& display_rect(
      const ::ui::ozone::mojom::OverlaySurfaceCandidatePtr& input) {
    return input->display_rect;
  }

  static const decltype(::ui::ozone::mojom::OverlaySurfaceCandidate::crop_rect)& crop_rect(
      const ::ui::ozone::mojom::OverlaySurfaceCandidatePtr& input) {
    return input->crop_rect;
  }

  static const decltype(::ui::ozone::mojom::OverlaySurfaceCandidate::clip_rect)& clip_rect(
      const ::ui::ozone::mojom::OverlaySurfaceCandidatePtr& input) {
    return input->clip_rect;
  }

  static decltype(::ui::ozone::mojom::OverlaySurfaceCandidate::is_clipped) is_clipped(
      const ::ui::ozone::mojom::OverlaySurfaceCandidatePtr& input) {
    return input->is_clipped;
  }

  static decltype(::ui::ozone::mojom::OverlaySurfaceCandidate::plane_z_order) plane_z_order(
      const ::ui::ozone::mojom::OverlaySurfaceCandidatePtr& input) {
    return input->plane_z_order;
  }

  static decltype(::ui::ozone::mojom::OverlaySurfaceCandidate::overlay_handled) overlay_handled(
      const ::ui::ozone::mojom::OverlaySurfaceCandidatePtr& input) {
    return input->overlay_handled;
  }

  static bool Read(::ui::ozone::mojom::OverlaySurfaceCandidate::DataView input, ::ui::ozone::mojom::OverlaySurfaceCandidatePtr* output);
};

}  // namespace mojo

#endif  // UI_OZONE_PUBLIC_INTERFACES_OVERLAY_SURFACE_CANDIDATE_MOJOM_H_