// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "services/network/public/mojom/network_service.mojom-blink.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "services/network/public/mojom/network_service.mojom-shared-message-ids.h"
#include "mojo/public/cpp/bindings/lib/wtf_serialization.h"
#include "ipc/ipc_message_utils.h"
#include "mojo/public/cpp/base/file_mojom_traits.h"
#include "mojo/public/cpp/base/file_path_mojom_traits.h"
#include "mojo/public/cpp/base/time_mojom_traits.h"
#include "third_party/blink/renderer/platform/mojo/canonical_cookie_mojom_traits.h"
#include "third_party/blink/renderer/platform/mojo/kurl_struct_traits.h"
namespace network {
namespace mojom {
namespace blink {
DnsOverHttpsServer::DnsOverHttpsServer()
    : url(),
      use_posts(false) {}

DnsOverHttpsServer::DnsOverHttpsServer(
    const ::blink::KURL& url_in,
    bool use_posts_in)
    : url(std::move(url_in)),
      use_posts(std::move(use_posts_in)) {}

DnsOverHttpsServer::~DnsOverHttpsServer() = default;

bool DnsOverHttpsServer::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
HttpAuthStaticParams::HttpAuthStaticParams()
    : supported_schemes(),
      gssapi_library_name(),
      allow_gssapi_library_load(true) {}

HttpAuthStaticParams::HttpAuthStaticParams(
    const WTF::Vector<WTF::String>& supported_schemes_in,
    const WTF::String& gssapi_library_name_in,
    bool allow_gssapi_library_load_in)
    : supported_schemes(std::move(supported_schemes_in)),
      gssapi_library_name(std::move(gssapi_library_name_in)),
      allow_gssapi_library_load(std::move(allow_gssapi_library_load_in)) {}

HttpAuthStaticParams::~HttpAuthStaticParams() = default;

bool HttpAuthStaticParams::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
HttpAuthDynamicParams::HttpAuthDynamicParams()
    : server_whitelist(),
      delegate_whitelist(),
      negotiate_disable_cname_lookup(false),
      enable_negotiate_port(true),
      ntlm_v2_enabled(false),
      android_negotiate_account_type() {}

HttpAuthDynamicParams::HttpAuthDynamicParams(
    const WTF::String& server_whitelist_in,
    const WTF::String& delegate_whitelist_in,
    bool negotiate_disable_cname_lookup_in,
    bool enable_negotiate_port_in,
    bool ntlm_v2_enabled_in,
    const WTF::String& android_negotiate_account_type_in)
    : server_whitelist(std::move(server_whitelist_in)),
      delegate_whitelist(std::move(delegate_whitelist_in)),
      negotiate_disable_cname_lookup(std::move(negotiate_disable_cname_lookup_in)),
      enable_negotiate_port(std::move(enable_negotiate_port_in)),
      ntlm_v2_enabled(std::move(ntlm_v2_enabled_in)),
      android_negotiate_account_type(std::move(android_negotiate_account_type_in)) {}

HttpAuthDynamicParams::~HttpAuthDynamicParams() = default;
size_t HttpAuthDynamicParams::Hash(size_t seed) const {
  seed = mojo::internal::WTFHash(seed, this->server_whitelist);
  seed = mojo::internal::WTFHash(seed, this->delegate_whitelist);
  seed = mojo::internal::WTFHash(seed, this->negotiate_disable_cname_lookup);
  seed = mojo::internal::WTFHash(seed, this->enable_negotiate_port);
  seed = mojo::internal::WTFHash(seed, this->ntlm_v2_enabled);
  seed = mojo::internal::WTFHash(seed, this->android_negotiate_account_type);
  return seed;
}

bool HttpAuthDynamicParams::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
CryptConfig::CryptConfig()
    : store(),
      product_name(),
      should_use_preference(),
      user_data_path() {}

CryptConfig::CryptConfig(
    const WTF::String& store_in,
    const WTF::String& product_name_in,
    bool should_use_preference_in,
    const base::FilePath& user_data_path_in)
    : store(std::move(store_in)),
      product_name(std::move(product_name_in)),
      should_use_preference(std::move(should_use_preference_in)),
      user_data_path(std::move(user_data_path_in)) {}

CryptConfig::~CryptConfig() = default;

bool CryptConfig::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char SSLPrivateKey::Name_[] = "network.mojom.SSLPrivateKey";

class SSLPrivateKey_Sign_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  SSLPrivateKey_Sign_ForwardToCallback(
      SSLPrivateKey::SignCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  SSLPrivateKey::SignCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(SSLPrivateKey_Sign_ForwardToCallback);
};

SSLPrivateKeyProxy::SSLPrivateKeyProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}
class SSLPrivateKeyProxy_Sign_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit SSLPrivateKeyProxy_Sign_Message(
      uint32_t message_flags
      , uint16_t param_algorithm
      , const WTF::Vector<uint8_t>& param_input
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kSSLPrivateKey_Sign_Name,
          message_flags)
      , param_algorithm_(std::move(param_algorithm))
      , param_input_(std::move(param_input)){}
  ~SSLPrivateKeyProxy_Sign_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      uint16_t param_algorithm, const WTF::Vector<uint8_t>& param_input) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<SSLPrivateKeyProxy_Sign_Message>(
          kFlags
          , std::move(param_algorithm)
          , std::move(param_input)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kSSLPrivateKey_Sign_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::SSLPrivateKey_Sign_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->algorithm = param_algorithm;
  typename decltype(params->input)::BaseType::BufferWriter
      input_writer;
  const mojo::internal::ContainerValidateParams input_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
      param_input, buffer, &input_writer, &input_validate_params,
      &serialization_context);
  params->input.Set(
      input_writer.is_null() ? nullptr : input_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->input.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null input in SSLPrivateKey.Sign request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(SSLPrivateKey* impl, SSLPrivateKey::SignCallback callback) {
    impl->Sign(
        std::move(param_algorithm_),
        std::move(param_input_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::SSLPrivateKey_Sign_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->algorithm = param_algorithm_;
  typename decltype(params->input)::BaseType::BufferWriter
      input_writer;
  const mojo::internal::ContainerValidateParams input_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
      param_input_, buffer, &input_writer, &input_validate_params,
      serialization_context);
  params->input.Set(
      input_writer.is_null() ? nullptr : input_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->input.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null input in SSLPrivateKey.Sign request");
  }
  uint16_t param_algorithm_;
  WTF::Vector<uint8_t> param_input_;

  DISALLOW_COPY_AND_ASSIGN(SSLPrivateKeyProxy_Sign_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
SSLPrivateKeyProxy_Sign_Message::kMessageTag = {};

void SSLPrivateKeyProxy::Sign(
    uint16_t in_algorithm, const WTF::Vector<uint8_t>& in_input, SignCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::SSLPrivateKey::Sign");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = SSLPrivateKeyProxy_Sign_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_algorithm), std::move(in_input));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(SSLPrivateKey::Name_);
  message.set_method_name("Sign");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new SSLPrivateKey_Sign_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class SSLPrivateKey_Sign_ProxyToResponder {
 public:
  static SSLPrivateKey::SignCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<SSLPrivateKey_Sign_ProxyToResponder> proxy(
        new SSLPrivateKey_Sign_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&SSLPrivateKey_Sign_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~SSLPrivateKey_Sign_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  SSLPrivateKey_Sign_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "SSLPrivateKey::SignCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      int32_t in_net_error, const WTF::Vector<uint8_t>& in_signature);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(SSLPrivateKey_Sign_ProxyToResponder);
};
class SSLPrivateKey_Sign_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit SSLPrivateKey_Sign_Response_Message(
      uint32_t message_flags
      , int32_t param_net_error
      , const WTF::Vector<uint8_t>& param_signature
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kSSLPrivateKey_Sign_Name,
          message_flags)
      , param_net_error_(std::move(param_net_error))
      , param_signature_(std::move(param_signature)){}
  ~SSLPrivateKey_Sign_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      int32_t param_net_error, const WTF::Vector<uint8_t>& param_signature) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<SSLPrivateKey_Sign_Response_Message>(
          kFlags
          , std::move(param_net_error)
          , std::move(param_signature)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kSSLPrivateKey_Sign_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::SSLPrivateKey_Sign_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->net_error = param_net_error;
  typename decltype(params->signature)::BaseType::BufferWriter
      signature_writer;
  const mojo::internal::ContainerValidateParams signature_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
      param_signature, buffer, &signature_writer, &signature_validate_params,
      &serialization_context);
  params->signature.Set(
      signature_writer.is_null() ? nullptr : signature_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->signature.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null signature in SSLPrivateKey.Sign response");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(SSLPrivateKey::SignCallback* callback) {
    std::move(*callback).Run(
        std::move(param_net_error_), 
        std::move(param_signature_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::SSLPrivateKey_Sign_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->net_error = param_net_error_;
  typename decltype(params->signature)::BaseType::BufferWriter
      signature_writer;
  const mojo::internal::ContainerValidateParams signature_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
      param_signature_, buffer, &signature_writer, &signature_validate_params,
      serialization_context);
  params->signature.Set(
      signature_writer.is_null() ? nullptr : signature_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->signature.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null signature in SSLPrivateKey.Sign response");
  }
  int32_t param_net_error_;
  WTF::Vector<uint8_t> param_signature_;

  DISALLOW_COPY_AND_ASSIGN(SSLPrivateKey_Sign_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
SSLPrivateKey_Sign_Response_Message::kMessageTag = {};

bool SSLPrivateKey_Sign_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::SSLPrivateKey::SignCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<SSLPrivateKey_Sign_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::SSLPrivateKey_Sign_ResponseParams_Data* params =
      reinterpret_cast<
          internal::SSLPrivateKey_Sign_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  int32_t p_net_error{};
  WTF::Vector<uint8_t> p_signature{};
  SSLPrivateKey_Sign_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_net_error = input_data_view.net_error();
  if (!input_data_view.ReadSignature(&p_signature))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "SSLPrivateKey::Sign response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_net_error), 
std::move(p_signature));
  return true;
}

void SSLPrivateKey_Sign_ProxyToResponder::Run(
    int32_t in_net_error, const WTF::Vector<uint8_t>& in_signature) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = SSLPrivateKey_Sign_Response_Message::Build(kSerialize, is_sync_, std::move(in_net_error), std::move(in_signature));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::SSLPrivateKey::SignCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(SSLPrivateKey::Name_);
  message.set_method_name("Sign");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool SSLPrivateKeyStubDispatch::Accept(
    SSLPrivateKey* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kSSLPrivateKey_Sign_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::SSLPrivateKey::Sign",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

// static
bool SSLPrivateKeyStubDispatch::AcceptWithResponder(
    SSLPrivateKey* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kSSLPrivateKey_Sign_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::SSLPrivateKey::Sign",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            SSLPrivateKeyProxy_Sign_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          SSLPrivateKey::SignCallback callback =
              SSLPrivateKey_Sign_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::SSLPrivateKey_Sign_Params_Data* params =
          reinterpret_cast<
              internal::SSLPrivateKey_Sign_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint16_t p_algorithm{};
      WTF::Vector<uint8_t> p_input{};
      SSLPrivateKey_Sign_ParamsDataView input_data_view(params, &serialization_context);
      
      p_algorithm = input_data_view.algorithm();
      if (!input_data_view.ReadInput(&p_input))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "SSLPrivateKey::Sign deserializer");
        return false;
      }
      SSLPrivateKey::SignCallback callback =
          SSLPrivateKey_Sign_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->Sign(
std::move(p_algorithm), 
std::move(p_input), std::move(callback));
      return true;
    }
  }
  return false;
}

bool SSLPrivateKeyRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "SSLPrivateKey RequestValidator");

  switch (message->header()->name) {
    case internal::kSSLPrivateKey_Sign_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::SSLPrivateKey_Sign_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool SSLPrivateKeyResponseValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "SSLPrivateKey ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kSSLPrivateKey_Sign_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::SSLPrivateKey_Sign_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
void SSLPrivateKeyInterceptorForTesting::Sign(uint16_t algorithm, const WTF::Vector<uint8_t>& input, SignCallback callback) {
  GetForwardingInterface()->Sign(std::move(algorithm), std::move(input), std::move(callback));
}
SSLPrivateKeyAsyncWaiter::SSLPrivateKeyAsyncWaiter(
    SSLPrivateKey* proxy) : proxy_(proxy) {}

SSLPrivateKeyAsyncWaiter::~SSLPrivateKeyAsyncWaiter() = default;

void SSLPrivateKeyAsyncWaiter::Sign(
    uint16_t algorithm, const WTF::Vector<uint8_t>& input, int32_t* out_net_error, WTF::Vector<uint8_t>* out_signature) {
  base::RunLoop loop;
  proxy_->Sign(std::move(algorithm),std::move(input),
      base::BindOnce(
          [](base::RunLoop* loop,
             int32_t* out_net_error
,
             WTF::Vector<uint8_t>* out_signature
,
             int32_t net_error,
             const WTF::Vector<uint8_t>& signature) {*out_net_error = std::move(net_error);*out_signature = std::move(signature);
            loop->Quit();
          },
          &loop,
          out_net_error,
          out_signature));
  loop.Run();
}

const char AuthChallengeResponder::Name_[] = "network.mojom.AuthChallengeResponder";

AuthChallengeResponderProxy::AuthChallengeResponderProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}
class AuthChallengeResponderProxy_OnAuthCredentials_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit AuthChallengeResponderProxy_OnAuthCredentials_Message(
      uint32_t message_flags
      , ::network::mojom::blink::AuthCredentialsPtr param_credentials
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kAuthChallengeResponder_OnAuthCredentials_Name,
          message_flags)
      , param_credentials_(std::move(param_credentials)){}
  ~AuthChallengeResponderProxy_OnAuthCredentials_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::blink::AuthCredentialsPtr param_credentials) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<AuthChallengeResponderProxy_OnAuthCredentials_Message>(
          kFlags
          , std::move(param_credentials)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kAuthChallengeResponder_OnAuthCredentials_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::AuthChallengeResponder_OnAuthCredentials_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->credentials)::BaseType::BufferWriter
      credentials_writer;
  mojo::internal::Serialize<::network::mojom::AuthCredentialsDataView>(
      param_credentials, buffer, &credentials_writer, &serialization_context);
  params->credentials.Set(
      credentials_writer.is_null() ? nullptr : credentials_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(AuthChallengeResponder* impl) {
    impl->OnAuthCredentials(
        std::move(param_credentials_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::AuthChallengeResponder_OnAuthCredentials_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->credentials)::BaseType::BufferWriter
      credentials_writer;
  mojo::internal::Serialize<::network::mojom::AuthCredentialsDataView>(
      param_credentials_, buffer, &credentials_writer, serialization_context);
  params->credentials.Set(
      credentials_writer.is_null() ? nullptr : credentials_writer.data());
  }
  ::network::mojom::blink::AuthCredentialsPtr param_credentials_;

  DISALLOW_COPY_AND_ASSIGN(AuthChallengeResponderProxy_OnAuthCredentials_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
AuthChallengeResponderProxy_OnAuthCredentials_Message::kMessageTag = {};

void AuthChallengeResponderProxy::OnAuthCredentials(
    ::network::mojom::blink::AuthCredentialsPtr in_credentials) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::AuthChallengeResponder::OnAuthCredentials");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = AuthChallengeResponderProxy_OnAuthCredentials_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_credentials));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(AuthChallengeResponder::Name_);
  message.set_method_name("OnAuthCredentials");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool AuthChallengeResponderStubDispatch::Accept(
    AuthChallengeResponder* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kAuthChallengeResponder_OnAuthCredentials_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::AuthChallengeResponder::OnAuthCredentials",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            AuthChallengeResponderProxy_OnAuthCredentials_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::AuthChallengeResponder_OnAuthCredentials_Params_Data* params =
          reinterpret_cast<internal::AuthChallengeResponder_OnAuthCredentials_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::blink::AuthCredentialsPtr p_credentials{};
      AuthChallengeResponder_OnAuthCredentials_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadCredentials(&p_credentials))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "AuthChallengeResponder::OnAuthCredentials deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnAuthCredentials(
std::move(p_credentials));
      return true;
    }
  }
  return false;
}

// static
bool AuthChallengeResponderStubDispatch::AcceptWithResponder(
    AuthChallengeResponder* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kAuthChallengeResponder_OnAuthCredentials_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::AuthChallengeResponder::OnAuthCredentials",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool AuthChallengeResponderRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "AuthChallengeResponder RequestValidator");

  switch (message->header()->name) {
    case internal::kAuthChallengeResponder_OnAuthCredentials_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::AuthChallengeResponder_OnAuthCredentials_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void AuthChallengeResponderInterceptorForTesting::OnAuthCredentials(::network::mojom::blink::AuthCredentialsPtr credentials) {
  GetForwardingInterface()->OnAuthCredentials(std::move(credentials));
}
AuthChallengeResponderAsyncWaiter::AuthChallengeResponderAsyncWaiter(
    AuthChallengeResponder* proxy) : proxy_(proxy) {}

AuthChallengeResponderAsyncWaiter::~AuthChallengeResponderAsyncWaiter() = default;


const char NetworkServiceClient::Name_[] = "network.mojom.NetworkServiceClient";

class NetworkServiceClient_OnCertificateRequested_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkServiceClient_OnCertificateRequested_ForwardToCallback(
      NetworkServiceClient::OnCertificateRequestedCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkServiceClient::OnCertificateRequestedCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClient_OnCertificateRequested_ForwardToCallback);
};

class NetworkServiceClient_OnSSLCertificateError_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkServiceClient_OnSSLCertificateError_ForwardToCallback(
      NetworkServiceClient::OnSSLCertificateErrorCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkServiceClient::OnSSLCertificateErrorCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClient_OnSSLCertificateError_ForwardToCallback);
};

class NetworkServiceClient_OnFileUploadRequested_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkServiceClient_OnFileUploadRequested_ForwardToCallback(
      NetworkServiceClient::OnFileUploadRequestedCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkServiceClient::OnFileUploadRequestedCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClient_OnFileUploadRequested_ForwardToCallback);
};

NetworkServiceClientProxy::NetworkServiceClientProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void NetworkServiceClientProxy::OnAuthRequired(
    uint32_t in_process_id, uint32_t in_routing_id, uint32_t in_request_id, const ::blink::KURL& in_url, const ::blink::KURL& in_site_for_cookies, bool in_first_auth_attempt, ::network::mojom::blink::AuthChallengeInfoPtr in_auth_info, int32_t in_resource_type, ::network::mojom::blink::URLResponseHeadPtr in_head, AuthChallengeResponderPtr in_auth_challenge_responder) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkServiceClient::OnAuthRequired");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kNetworkServiceClient_OnAuthRequired_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkServiceClient_OnAuthRequired_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->process_id = in_process_id;
  params->routing_id = in_routing_id;
  params->request_id = in_request_id;
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      in_url, buffer, &url_writer, &serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkServiceClient.OnAuthRequired request");
  typename decltype(params->site_for_cookies)::BaseType::BufferWriter
      site_for_cookies_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      in_site_for_cookies, buffer, &site_for_cookies_writer, &serialization_context);
  params->site_for_cookies.Set(
      site_for_cookies_writer.is_null() ? nullptr : site_for_cookies_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->site_for_cookies.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null site_for_cookies in NetworkServiceClient.OnAuthRequired request");
  params->first_auth_attempt = in_first_auth_attempt;
  typename decltype(params->auth_info)::BaseType::BufferWriter
      auth_info_writer;
  mojo::internal::Serialize<::network::mojom::AuthChallengeInfoDataView>(
      in_auth_info, buffer, &auth_info_writer, &serialization_context);
  params->auth_info.Set(
      auth_info_writer.is_null() ? nullptr : auth_info_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->auth_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null auth_info in NetworkServiceClient.OnAuthRequired request");
  params->resource_type = in_resource_type;
  typename decltype(params->head)::BaseType::BufferWriter
      head_writer;
  mojo::internal::Serialize<::network::mojom::URLResponseHeadDataView>(
      in_head, buffer, &head_writer, &serialization_context);
  params->head.Set(
      head_writer.is_null() ? nullptr : head_writer.data());
  mojo::internal::Serialize<::network::mojom::AuthChallengeResponderPtrDataView>(
      in_auth_challenge_responder, &params->auth_challenge_responder, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->auth_challenge_responder),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid auth_challenge_responder in NetworkServiceClient.OnAuthRequired request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkServiceClient::Name_);
  message.set_method_name("OnAuthRequired");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkServiceClientProxy_OnCertificateRequested_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceClientProxy_OnCertificateRequested_Message(
      uint32_t message_flags
      , uint32_t param_process_id
      , uint32_t param_routing_id
      , uint32_t param_request_id
      , ::network::mojom::blink::SSLCertRequestInfoPtr param_cert_info
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkServiceClient_OnCertificateRequested_Name,
          message_flags)
      , param_process_id_(std::move(param_process_id))
      , param_routing_id_(std::move(param_routing_id))
      , param_request_id_(std::move(param_request_id))
      , param_cert_info_(std::move(param_cert_info)){}
  ~NetworkServiceClientProxy_OnCertificateRequested_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      uint32_t param_process_id, uint32_t param_routing_id, uint32_t param_request_id, ::network::mojom::blink::SSLCertRequestInfoPtr param_cert_info) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceClientProxy_OnCertificateRequested_Message>(
          kFlags
          , std::move(param_process_id)
          , std::move(param_routing_id)
          , std::move(param_request_id)
          , std::move(param_cert_info)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkServiceClient_OnCertificateRequested_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkServiceClient_OnCertificateRequested_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->process_id = param_process_id;
  params->routing_id = param_routing_id;
  params->request_id = param_request_id;
  typename decltype(params->cert_info)::BaseType::BufferWriter
      cert_info_writer;
  mojo::internal::Serialize<::network::mojom::SSLCertRequestInfoDataView>(
      param_cert_info, buffer, &cert_info_writer, &serialization_context);
  params->cert_info.Set(
      cert_info_writer.is_null() ? nullptr : cert_info_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->cert_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null cert_info in NetworkServiceClient.OnCertificateRequested request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkServiceClient* impl, NetworkServiceClient::OnCertificateRequestedCallback callback) {
    impl->OnCertificateRequested(
        std::move(param_process_id_),
        std::move(param_routing_id_),
        std::move(param_request_id_),
        std::move(param_cert_info_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkServiceClient_OnCertificateRequested_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->process_id = param_process_id_;
  params->routing_id = param_routing_id_;
  params->request_id = param_request_id_;
  typename decltype(params->cert_info)::BaseType::BufferWriter
      cert_info_writer;
  mojo::internal::Serialize<::network::mojom::SSLCertRequestInfoDataView>(
      param_cert_info_, buffer, &cert_info_writer, serialization_context);
  params->cert_info.Set(
      cert_info_writer.is_null() ? nullptr : cert_info_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->cert_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null cert_info in NetworkServiceClient.OnCertificateRequested request");
  }
  uint32_t param_process_id_;
  uint32_t param_routing_id_;
  uint32_t param_request_id_;
  ::network::mojom::blink::SSLCertRequestInfoPtr param_cert_info_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClientProxy_OnCertificateRequested_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceClientProxy_OnCertificateRequested_Message::kMessageTag = {};

void NetworkServiceClientProxy::OnCertificateRequested(
    uint32_t in_process_id, uint32_t in_routing_id, uint32_t in_request_id, ::network::mojom::blink::SSLCertRequestInfoPtr in_cert_info, OnCertificateRequestedCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkServiceClient::OnCertificateRequested");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceClientProxy_OnCertificateRequested_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_process_id), std::move(in_routing_id), std::move(in_request_id), std::move(in_cert_info));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkServiceClient::Name_);
  message.set_method_name("OnCertificateRequested");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkServiceClient_OnCertificateRequested_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void NetworkServiceClientProxy::OnSSLCertificateError(
    uint32_t in_process_id, uint32_t in_routing_id, uint32_t in_request_id, int32_t in_resource_type, const ::blink::KURL& in_url, ::network::mojom::blink::SSLInfoPtr in_ssl_info, bool in_fatal, OnSSLCertificateErrorCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkServiceClient::OnSSLCertificateError");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kNetworkServiceClient_OnSSLCertificateError_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkServiceClient_OnSSLCertificateError_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->process_id = in_process_id;
  params->routing_id = in_routing_id;
  params->request_id = in_request_id;
  params->resource_type = in_resource_type;
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      in_url, buffer, &url_writer, &serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkServiceClient.OnSSLCertificateError request");
  typename decltype(params->ssl_info)::BaseType::BufferWriter
      ssl_info_writer;
  mojo::internal::Serialize<::network::mojom::SSLInfoDataView>(
      in_ssl_info, buffer, &ssl_info_writer, &serialization_context);
  params->ssl_info.Set(
      ssl_info_writer.is_null() ? nullptr : ssl_info_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->ssl_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null ssl_info in NetworkServiceClient.OnSSLCertificateError request");
  params->fatal = in_fatal;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkServiceClient::Name_);
  message.set_method_name("OnSSLCertificateError");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkServiceClient_OnSSLCertificateError_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkServiceClientProxy_OnFileUploadRequested_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceClientProxy_OnFileUploadRequested_Message(
      uint32_t message_flags
      , uint32_t param_process_id
      , bool param_async
      , const WTF::Vector<base::FilePath>& param_file_paths
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkServiceClient_OnFileUploadRequested_Name,
          message_flags)
      , param_process_id_(std::move(param_process_id))
      , param_async_(std::move(param_async))
      , param_file_paths_(std::move(param_file_paths)){}
  ~NetworkServiceClientProxy_OnFileUploadRequested_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      uint32_t param_process_id, bool param_async, const WTF::Vector<base::FilePath>& param_file_paths) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceClientProxy_OnFileUploadRequested_Message>(
          kFlags
          , std::move(param_process_id)
          , std::move(param_async)
          , std::move(param_file_paths)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkServiceClient_OnFileUploadRequested_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkServiceClient_OnFileUploadRequested_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->process_id = param_process_id;
  params->async = param_async;
  typename decltype(params->file_paths)::BaseType::BufferWriter
      file_paths_writer;
  const mojo::internal::ContainerValidateParams file_paths_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::mojo_base::mojom::FilePathDataView>>(
      param_file_paths, buffer, &file_paths_writer, &file_paths_validate_params,
      &serialization_context);
  params->file_paths.Set(
      file_paths_writer.is_null() ? nullptr : file_paths_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->file_paths.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null file_paths in NetworkServiceClient.OnFileUploadRequested request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkServiceClient* impl, NetworkServiceClient::OnFileUploadRequestedCallback callback) {
    impl->OnFileUploadRequested(
        std::move(param_process_id_),
        std::move(param_async_),
        std::move(param_file_paths_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkServiceClient_OnFileUploadRequested_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->process_id = param_process_id_;
  params->async = param_async_;
  typename decltype(params->file_paths)::BaseType::BufferWriter
      file_paths_writer;
  const mojo::internal::ContainerValidateParams file_paths_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::mojo_base::mojom::FilePathDataView>>(
      param_file_paths_, buffer, &file_paths_writer, &file_paths_validate_params,
      serialization_context);
  params->file_paths.Set(
      file_paths_writer.is_null() ? nullptr : file_paths_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->file_paths.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null file_paths in NetworkServiceClient.OnFileUploadRequested request");
  }
  uint32_t param_process_id_;
  bool param_async_;
  WTF::Vector<base::FilePath> param_file_paths_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClientProxy_OnFileUploadRequested_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceClientProxy_OnFileUploadRequested_Message::kMessageTag = {};

void NetworkServiceClientProxy::OnFileUploadRequested(
    uint32_t in_process_id, bool in_async, const WTF::Vector<base::FilePath>& in_file_paths, OnFileUploadRequestedCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkServiceClient::OnFileUploadRequested");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceClientProxy_OnFileUploadRequested_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_process_id), std::move(in_async), std::move(in_file_paths));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkServiceClient::Name_);
  message.set_method_name("OnFileUploadRequested");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkServiceClient_OnFileUploadRequested_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void NetworkServiceClientProxy::OnCookieChange(
    int32_t in_process_id, int32_t in_routing_id, const ::blink::KURL& in_url, const ::blink::KURL& in_frame_url, const ::blink::WebCanonicalCookie& in_cookie, bool in_blocked_by_policy) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkServiceClient::OnCookieChange");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kNetworkServiceClient_OnCookieChange_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkServiceClient_OnCookieChange_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->process_id = in_process_id;
  params->routing_id = in_routing_id;
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      in_url, buffer, &url_writer, &serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkServiceClient.OnCookieChange request");
  typename decltype(params->frame_url)::BaseType::BufferWriter
      frame_url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      in_frame_url, buffer, &frame_url_writer, &serialization_context);
  params->frame_url.Set(
      frame_url_writer.is_null() ? nullptr : frame_url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->frame_url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null frame_url in NetworkServiceClient.OnCookieChange request");
  typename decltype(params->cookie)::BaseType::BufferWriter
      cookie_writer;
  mojo::internal::Serialize<::network::mojom::CanonicalCookieDataView>(
      in_cookie, buffer, &cookie_writer, &serialization_context);
  params->cookie.Set(
      cookie_writer.is_null() ? nullptr : cookie_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->cookie.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null cookie in NetworkServiceClient.OnCookieChange request");
  params->blocked_by_policy = in_blocked_by_policy;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkServiceClient::Name_);
  message.set_method_name("OnCookieChange");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void NetworkServiceClientProxy::OnCookiesRead(
    int32_t in_process_id, int32_t in_routing_id, const ::blink::KURL& in_url, const ::blink::KURL& in_frame_url, const WTF::Vector<::blink::WebCanonicalCookie>& in_cookie_list, bool in_blocked_by_policy) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkServiceClient::OnCookiesRead");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kNetworkServiceClient_OnCookiesRead_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkServiceClient_OnCookiesRead_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->process_id = in_process_id;
  params->routing_id = in_routing_id;
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      in_url, buffer, &url_writer, &serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkServiceClient.OnCookiesRead request");
  typename decltype(params->frame_url)::BaseType::BufferWriter
      frame_url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      in_frame_url, buffer, &frame_url_writer, &serialization_context);
  params->frame_url.Set(
      frame_url_writer.is_null() ? nullptr : frame_url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->frame_url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null frame_url in NetworkServiceClient.OnCookiesRead request");
  typename decltype(params->cookie_list)::BaseType::BufferWriter
      cookie_list_writer;
  const mojo::internal::ContainerValidateParams cookie_list_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::network::mojom::CanonicalCookieDataView>>(
      in_cookie_list, buffer, &cookie_list_writer, &cookie_list_validate_params,
      &serialization_context);
  params->cookie_list.Set(
      cookie_list_writer.is_null() ? nullptr : cookie_list_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->cookie_list.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null cookie_list in NetworkServiceClient.OnCookiesRead request");
  params->blocked_by_policy = in_blocked_by_policy;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkServiceClient::Name_);
  message.set_method_name("OnCookiesRead");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkServiceClient_OnCertificateRequested_ProxyToResponder {
 public:
  static NetworkServiceClient::OnCertificateRequestedCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkServiceClient_OnCertificateRequested_ProxyToResponder> proxy(
        new NetworkServiceClient_OnCertificateRequested_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkServiceClient_OnCertificateRequested_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkServiceClient_OnCertificateRequested_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkServiceClient_OnCertificateRequested_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkServiceClient::OnCertificateRequestedCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      ::network::mojom::blink::X509CertificatePtr in_x509_certificate, const WTF::Vector<uint16_t>& in_algorithm_preferences, SSLPrivateKeyPtr in_ssl_private_key, bool in_cancel_certificate_selection);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClient_OnCertificateRequested_ProxyToResponder);
};
class NetworkServiceClient_OnCertificateRequested_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceClient_OnCertificateRequested_Response_Message(
      uint32_t message_flags
      , ::network::mojom::blink::X509CertificatePtr param_x509_certificate
      , const WTF::Vector<uint16_t>& param_algorithm_preferences
      , SSLPrivateKeyPtr param_ssl_private_key
      , bool param_cancel_certificate_selection
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkServiceClient_OnCertificateRequested_Name,
          message_flags)
      , param_x509_certificate_(std::move(param_x509_certificate))
      , param_algorithm_preferences_(std::move(param_algorithm_preferences))
      , param_ssl_private_key_(param_ssl_private_key.PassInterface())
      , param_cancel_certificate_selection_(std::move(param_cancel_certificate_selection)){}
  ~NetworkServiceClient_OnCertificateRequested_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      ::network::mojom::blink::X509CertificatePtr param_x509_certificate, const WTF::Vector<uint16_t>& param_algorithm_preferences, SSLPrivateKeyPtr param_ssl_private_key, bool param_cancel_certificate_selection) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceClient_OnCertificateRequested_Response_Message>(
          kFlags
          , std::move(param_x509_certificate)
          , std::move(param_algorithm_preferences)
          , std::move(param_ssl_private_key)
          , std::move(param_cancel_certificate_selection)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkServiceClient_OnCertificateRequested_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkServiceClient_OnCertificateRequested_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->x509_certificate)::BaseType::BufferWriter
      x509_certificate_writer;
  mojo::internal::Serialize<::network::mojom::X509CertificateDataView>(
      param_x509_certificate, buffer, &x509_certificate_writer, &serialization_context);
  params->x509_certificate.Set(
      x509_certificate_writer.is_null() ? nullptr : x509_certificate_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->x509_certificate.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null x509_certificate in NetworkServiceClient.OnCertificateRequested response");
  typename decltype(params->algorithm_preferences)::BaseType::BufferWriter
      algorithm_preferences_writer;
  const mojo::internal::ContainerValidateParams algorithm_preferences_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint16_t>>(
      param_algorithm_preferences, buffer, &algorithm_preferences_writer, &algorithm_preferences_validate_params,
      &serialization_context);
  params->algorithm_preferences.Set(
      algorithm_preferences_writer.is_null() ? nullptr : algorithm_preferences_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->algorithm_preferences.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null algorithm_preferences in NetworkServiceClient.OnCertificateRequested response");
  mojo::internal::Serialize<::network::mojom::SSLPrivateKeyPtrDataView>(
      param_ssl_private_key, &params->ssl_private_key, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->ssl_private_key),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid ssl_private_key in NetworkServiceClient.OnCertificateRequested response");
  params->cancel_certificate_selection = param_cancel_certificate_selection;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkServiceClient::OnCertificateRequestedCallback* callback) {
    std::move(*callback).Run(
        std::move(param_x509_certificate_), 
        std::move(param_algorithm_preferences_), 
        SSLPrivateKeyPtr(std::move(param_ssl_private_key_)), 
        std::move(param_cancel_certificate_selection_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkServiceClient_OnCertificateRequested_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->x509_certificate)::BaseType::BufferWriter
      x509_certificate_writer;
  mojo::internal::Serialize<::network::mojom::X509CertificateDataView>(
      param_x509_certificate_, buffer, &x509_certificate_writer, serialization_context);
  params->x509_certificate.Set(
      x509_certificate_writer.is_null() ? nullptr : x509_certificate_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->x509_certificate.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null x509_certificate in NetworkServiceClient.OnCertificateRequested response");
  typename decltype(params->algorithm_preferences)::BaseType::BufferWriter
      algorithm_preferences_writer;
  const mojo::internal::ContainerValidateParams algorithm_preferences_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint16_t>>(
      param_algorithm_preferences_, buffer, &algorithm_preferences_writer, &algorithm_preferences_validate_params,
      serialization_context);
  params->algorithm_preferences.Set(
      algorithm_preferences_writer.is_null() ? nullptr : algorithm_preferences_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->algorithm_preferences.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null algorithm_preferences in NetworkServiceClient.OnCertificateRequested response");
  mojo::internal::Serialize<::network::mojom::SSLPrivateKeyPtrDataView>(
      param_ssl_private_key_, &params->ssl_private_key, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->ssl_private_key),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid ssl_private_key in NetworkServiceClient.OnCertificateRequested response");
  params->cancel_certificate_selection = param_cancel_certificate_selection_;
  }
  ::network::mojom::blink::X509CertificatePtr param_x509_certificate_;
  WTF::Vector<uint16_t> param_algorithm_preferences_;
  SSLPrivateKeyPtrInfo param_ssl_private_key_;
  bool param_cancel_certificate_selection_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClient_OnCertificateRequested_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceClient_OnCertificateRequested_Response_Message::kMessageTag = {};

bool NetworkServiceClient_OnCertificateRequested_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkServiceClient::OnCertificateRequestedCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkServiceClient_OnCertificateRequested_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkServiceClient_OnCertificateRequested_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkServiceClient_OnCertificateRequested_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  ::network::mojom::blink::X509CertificatePtr p_x509_certificate{};
  WTF::Vector<uint16_t> p_algorithm_preferences{};
  SSLPrivateKeyPtr p_ssl_private_key{};
  bool p_cancel_certificate_selection{};
  NetworkServiceClient_OnCertificateRequested_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadX509Certificate(&p_x509_certificate))
    success = false;
  if (!input_data_view.ReadAlgorithmPreferences(&p_algorithm_preferences))
    success = false;
  p_ssl_private_key =
      input_data_view.TakeSslPrivateKey<decltype(p_ssl_private_key)>();
  p_cancel_certificate_selection = input_data_view.cancel_certificate_selection();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkServiceClient::OnCertificateRequested response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_x509_certificate), 
std::move(p_algorithm_preferences), 
std::move(p_ssl_private_key), 
std::move(p_cancel_certificate_selection));
  return true;
}

void NetworkServiceClient_OnCertificateRequested_ProxyToResponder::Run(
    ::network::mojom::blink::X509CertificatePtr in_x509_certificate, const WTF::Vector<uint16_t>& in_algorithm_preferences, SSLPrivateKeyPtr in_ssl_private_key, bool in_cancel_certificate_selection) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkServiceClient_OnCertificateRequested_Response_Message::Build(kSerialize, is_sync_, std::move(in_x509_certificate), std::move(in_algorithm_preferences), std::move(in_ssl_private_key), std::move(in_cancel_certificate_selection));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnCertificateRequestedCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkServiceClient::Name_);
  message.set_method_name("OnCertificateRequested");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkServiceClient_OnSSLCertificateError_ProxyToResponder {
 public:
  static NetworkServiceClient::OnSSLCertificateErrorCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkServiceClient_OnSSLCertificateError_ProxyToResponder> proxy(
        new NetworkServiceClient_OnSSLCertificateError_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkServiceClient_OnSSLCertificateError_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkServiceClient_OnSSLCertificateError_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkServiceClient_OnSSLCertificateError_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkServiceClient::OnSSLCertificateErrorCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      int32_t in_net_error);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClient_OnSSLCertificateError_ProxyToResponder);
};

bool NetworkServiceClient_OnSSLCertificateError_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkServiceClient::OnSSLCertificateErrorCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  int32_t p_net_error{};
  NetworkServiceClient_OnSSLCertificateError_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_net_error = input_data_view.net_error();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkServiceClient::OnSSLCertificateError response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_net_error));
  return true;
}

void NetworkServiceClient_OnSSLCertificateError_ProxyToResponder::Run(
    int32_t in_net_error) {
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kNetworkServiceClient_OnSSLCertificateError_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->net_error = in_net_error;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnSSLCertificateErrorCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkServiceClient::Name_);
  message.set_method_name("OnSSLCertificateError");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkServiceClient_OnFileUploadRequested_ProxyToResponder {
 public:
  static NetworkServiceClient::OnFileUploadRequestedCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkServiceClient_OnFileUploadRequested_ProxyToResponder> proxy(
        new NetworkServiceClient_OnFileUploadRequested_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkServiceClient_OnFileUploadRequested_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkServiceClient_OnFileUploadRequested_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkServiceClient_OnFileUploadRequested_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkServiceClient::OnFileUploadRequestedCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      int32_t in_net_error, WTF::Vector<base::File> in_files);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClient_OnFileUploadRequested_ProxyToResponder);
};
class NetworkServiceClient_OnFileUploadRequested_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceClient_OnFileUploadRequested_Response_Message(
      uint32_t message_flags
      , int32_t param_net_error
      , WTF::Vector<base::File> param_files
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkServiceClient_OnFileUploadRequested_Name,
          message_flags)
      , param_net_error_(std::move(param_net_error))
      , param_files_(std::move(param_files)){}
  ~NetworkServiceClient_OnFileUploadRequested_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      int32_t param_net_error, WTF::Vector<base::File> param_files) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceClient_OnFileUploadRequested_Response_Message>(
          kFlags
          , std::move(param_net_error)
          , std::move(param_files)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkServiceClient_OnFileUploadRequested_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkServiceClient_OnFileUploadRequested_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->net_error = param_net_error;
  typename decltype(params->files)::BaseType::BufferWriter
      files_writer;
  const mojo::internal::ContainerValidateParams files_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::mojo_base::mojom::FileDataView>>(
      param_files, buffer, &files_writer, &files_validate_params,
      &serialization_context);
  params->files.Set(
      files_writer.is_null() ? nullptr : files_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->files.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null files in NetworkServiceClient.OnFileUploadRequested response");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkServiceClient::OnFileUploadRequestedCallback* callback) {
    std::move(*callback).Run(
        std::move(param_net_error_), 
        std::move(param_files_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkServiceClient_OnFileUploadRequested_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->net_error = param_net_error_;
  typename decltype(params->files)::BaseType::BufferWriter
      files_writer;
  const mojo::internal::ContainerValidateParams files_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::mojo_base::mojom::FileDataView>>(
      param_files_, buffer, &files_writer, &files_validate_params,
      serialization_context);
  params->files.Set(
      files_writer.is_null() ? nullptr : files_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->files.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null files in NetworkServiceClient.OnFileUploadRequested response");
  }
  int32_t param_net_error_;
  WTF::Vector<base::File> param_files_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClient_OnFileUploadRequested_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceClient_OnFileUploadRequested_Response_Message::kMessageTag = {};

bool NetworkServiceClient_OnFileUploadRequested_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkServiceClient::OnFileUploadRequestedCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkServiceClient_OnFileUploadRequested_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkServiceClient_OnFileUploadRequested_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkServiceClient_OnFileUploadRequested_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  int32_t p_net_error{};
  WTF::Vector<base::File> p_files{};
  NetworkServiceClient_OnFileUploadRequested_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_net_error = input_data_view.net_error();
  if (!input_data_view.ReadFiles(&p_files))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkServiceClient::OnFileUploadRequested response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_net_error), 
std::move(p_files));
  return true;
}

void NetworkServiceClient_OnFileUploadRequested_ProxyToResponder::Run(
    int32_t in_net_error, WTF::Vector<base::File> in_files) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkServiceClient_OnFileUploadRequested_Response_Message::Build(kSerialize, is_sync_, std::move(in_net_error), std::move(in_files));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnFileUploadRequestedCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkServiceClient::Name_);
  message.set_method_name("OnFileUploadRequested");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool NetworkServiceClientStubDispatch::Accept(
    NetworkServiceClient* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kNetworkServiceClient_OnAuthRequired_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnAuthRequired",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::NetworkServiceClient_OnAuthRequired_Params_Data* params =
          reinterpret_cast<internal::NetworkServiceClient_OnAuthRequired_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_process_id{};
      uint32_t p_routing_id{};
      uint32_t p_request_id{};
      ::blink::KURL p_url{};
      ::blink::KURL p_site_for_cookies{};
      bool p_first_auth_attempt{};
      ::network::mojom::blink::AuthChallengeInfoPtr p_auth_info{};
      int32_t p_resource_type{};
      ::network::mojom::blink::URLResponseHeadPtr p_head{};
      AuthChallengeResponderPtr p_auth_challenge_responder{};
      NetworkServiceClient_OnAuthRequired_ParamsDataView input_data_view(params, &serialization_context);
      
      p_process_id = input_data_view.process_id();
      p_routing_id = input_data_view.routing_id();
      p_request_id = input_data_view.request_id();
      if (!input_data_view.ReadUrl(&p_url))
        success = false;
      if (!input_data_view.ReadSiteForCookies(&p_site_for_cookies))
        success = false;
      p_first_auth_attempt = input_data_view.first_auth_attempt();
      if (!input_data_view.ReadAuthInfo(&p_auth_info))
        success = false;
      p_resource_type = input_data_view.resource_type();
      if (!input_data_view.ReadHead(&p_head))
        success = false;
      p_auth_challenge_responder =
          input_data_view.TakeAuthChallengeResponder<decltype(p_auth_challenge_responder)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkServiceClient::OnAuthRequired deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnAuthRequired(
std::move(p_process_id), 
std::move(p_routing_id), 
std::move(p_request_id), 
std::move(p_url), 
std::move(p_site_for_cookies), 
std::move(p_first_auth_attempt), 
std::move(p_auth_info), 
std::move(p_resource_type), 
std::move(p_head), 
std::move(p_auth_challenge_responder));
      return true;
    }
    case internal::kNetworkServiceClient_OnCertificateRequested_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnCertificateRequested",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkServiceClient_OnSSLCertificateError_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnSSLCertificateError",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkServiceClient_OnFileUploadRequested_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnFileUploadRequested",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkServiceClient_OnCookieChange_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnCookieChange",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::NetworkServiceClient_OnCookieChange_Params_Data* params =
          reinterpret_cast<internal::NetworkServiceClient_OnCookieChange_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int32_t p_process_id{};
      int32_t p_routing_id{};
      ::blink::KURL p_url{};
      ::blink::KURL p_frame_url{};
      ::blink::WebCanonicalCookie p_cookie{};
      bool p_blocked_by_policy{};
      NetworkServiceClient_OnCookieChange_ParamsDataView input_data_view(params, &serialization_context);
      
      p_process_id = input_data_view.process_id();
      p_routing_id = input_data_view.routing_id();
      if (!input_data_view.ReadUrl(&p_url))
        success = false;
      if (!input_data_view.ReadFrameUrl(&p_frame_url))
        success = false;
      if (!input_data_view.ReadCookie(&p_cookie))
        success = false;
      p_blocked_by_policy = input_data_view.blocked_by_policy();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkServiceClient::OnCookieChange deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnCookieChange(
std::move(p_process_id), 
std::move(p_routing_id), 
std::move(p_url), 
std::move(p_frame_url), 
std::move(p_cookie), 
std::move(p_blocked_by_policy));
      return true;
    }
    case internal::kNetworkServiceClient_OnCookiesRead_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnCookiesRead",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::NetworkServiceClient_OnCookiesRead_Params_Data* params =
          reinterpret_cast<internal::NetworkServiceClient_OnCookiesRead_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int32_t p_process_id{};
      int32_t p_routing_id{};
      ::blink::KURL p_url{};
      ::blink::KURL p_frame_url{};
      WTF::Vector<::blink::WebCanonicalCookie> p_cookie_list{};
      bool p_blocked_by_policy{};
      NetworkServiceClient_OnCookiesRead_ParamsDataView input_data_view(params, &serialization_context);
      
      p_process_id = input_data_view.process_id();
      p_routing_id = input_data_view.routing_id();
      if (!input_data_view.ReadUrl(&p_url))
        success = false;
      if (!input_data_view.ReadFrameUrl(&p_frame_url))
        success = false;
      if (!input_data_view.ReadCookieList(&p_cookie_list))
        success = false;
      p_blocked_by_policy = input_data_view.blocked_by_policy();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkServiceClient::OnCookiesRead deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnCookiesRead(
std::move(p_process_id), 
std::move(p_routing_id), 
std::move(p_url), 
std::move(p_frame_url), 
std::move(p_cookie_list), 
std::move(p_blocked_by_policy));
      return true;
    }
  }
  return false;
}

// static
bool NetworkServiceClientStubDispatch::AcceptWithResponder(
    NetworkServiceClient* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kNetworkServiceClient_OnAuthRequired_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnAuthRequired",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkServiceClient_OnCertificateRequested_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnCertificateRequested",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceClientProxy_OnCertificateRequested_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkServiceClient::OnCertificateRequestedCallback callback =
              NetworkServiceClient_OnCertificateRequested_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkServiceClient_OnCertificateRequested_Params_Data* params =
          reinterpret_cast<
              internal::NetworkServiceClient_OnCertificateRequested_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_process_id{};
      uint32_t p_routing_id{};
      uint32_t p_request_id{};
      ::network::mojom::blink::SSLCertRequestInfoPtr p_cert_info{};
      NetworkServiceClient_OnCertificateRequested_ParamsDataView input_data_view(params, &serialization_context);
      
      p_process_id = input_data_view.process_id();
      p_routing_id = input_data_view.routing_id();
      p_request_id = input_data_view.request_id();
      if (!input_data_view.ReadCertInfo(&p_cert_info))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkServiceClient::OnCertificateRequested deserializer");
        return false;
      }
      NetworkServiceClient::OnCertificateRequestedCallback callback =
          NetworkServiceClient_OnCertificateRequested_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnCertificateRequested(
std::move(p_process_id), 
std::move(p_routing_id), 
std::move(p_request_id), 
std::move(p_cert_info), std::move(callback));
      return true;
    }
    case internal::kNetworkServiceClient_OnSSLCertificateError_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnSSLCertificateError",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      internal::NetworkServiceClient_OnSSLCertificateError_Params_Data* params =
          reinterpret_cast<
              internal::NetworkServiceClient_OnSSLCertificateError_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_process_id{};
      uint32_t p_routing_id{};
      uint32_t p_request_id{};
      int32_t p_resource_type{};
      ::blink::KURL p_url{};
      ::network::mojom::blink::SSLInfoPtr p_ssl_info{};
      bool p_fatal{};
      NetworkServiceClient_OnSSLCertificateError_ParamsDataView input_data_view(params, &serialization_context);
      
      p_process_id = input_data_view.process_id();
      p_routing_id = input_data_view.routing_id();
      p_request_id = input_data_view.request_id();
      p_resource_type = input_data_view.resource_type();
      if (!input_data_view.ReadUrl(&p_url))
        success = false;
      if (!input_data_view.ReadSslInfo(&p_ssl_info))
        success = false;
      p_fatal = input_data_view.fatal();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkServiceClient::OnSSLCertificateError deserializer");
        return false;
      }
      NetworkServiceClient::OnSSLCertificateErrorCallback callback =
          NetworkServiceClient_OnSSLCertificateError_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnSSLCertificateError(
std::move(p_process_id), 
std::move(p_routing_id), 
std::move(p_request_id), 
std::move(p_resource_type), 
std::move(p_url), 
std::move(p_ssl_info), 
std::move(p_fatal), std::move(callback));
      return true;
    }
    case internal::kNetworkServiceClient_OnFileUploadRequested_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnFileUploadRequested",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceClientProxy_OnFileUploadRequested_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkServiceClient::OnFileUploadRequestedCallback callback =
              NetworkServiceClient_OnFileUploadRequested_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkServiceClient_OnFileUploadRequested_Params_Data* params =
          reinterpret_cast<
              internal::NetworkServiceClient_OnFileUploadRequested_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_process_id{};
      bool p_async{};
      WTF::Vector<base::FilePath> p_file_paths{};
      NetworkServiceClient_OnFileUploadRequested_ParamsDataView input_data_view(params, &serialization_context);
      
      p_process_id = input_data_view.process_id();
      p_async = input_data_view.async();
      if (!input_data_view.ReadFilePaths(&p_file_paths))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkServiceClient::OnFileUploadRequested deserializer");
        return false;
      }
      NetworkServiceClient::OnFileUploadRequestedCallback callback =
          NetworkServiceClient_OnFileUploadRequested_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnFileUploadRequested(
std::move(p_process_id), 
std::move(p_async), 
std::move(p_file_paths), std::move(callback));
      return true;
    }
    case internal::kNetworkServiceClient_OnCookieChange_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnCookieChange",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkServiceClient_OnCookiesRead_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnCookiesRead",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool NetworkServiceClientRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "NetworkServiceClient RequestValidator");

  switch (message->header()->name) {
    case internal::kNetworkServiceClient_OnAuthRequired_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkServiceClient_OnAuthRequired_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkServiceClient_OnCertificateRequested_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkServiceClient_OnCertificateRequested_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkServiceClient_OnSSLCertificateError_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkServiceClient_OnSSLCertificateError_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkServiceClient_OnFileUploadRequested_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkServiceClient_OnFileUploadRequested_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkServiceClient_OnCookieChange_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkServiceClient_OnCookieChange_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkServiceClient_OnCookiesRead_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkServiceClient_OnCookiesRead_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool NetworkServiceClientResponseValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "NetworkServiceClient ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kNetworkServiceClient_OnCertificateRequested_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkServiceClient_OnCertificateRequested_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkServiceClient_OnSSLCertificateError_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkServiceClient_OnFileUploadRequested_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkServiceClient_OnFileUploadRequested_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
void NetworkServiceClientInterceptorForTesting::OnAuthRequired(uint32_t process_id, uint32_t routing_id, uint32_t request_id, const ::blink::KURL& url, const ::blink::KURL& site_for_cookies, bool first_auth_attempt, ::network::mojom::blink::AuthChallengeInfoPtr auth_info, int32_t resource_type, ::network::mojom::blink::URLResponseHeadPtr head, AuthChallengeResponderPtr auth_challenge_responder) {
  GetForwardingInterface()->OnAuthRequired(std::move(process_id), std::move(routing_id), std::move(request_id), std::move(url), std::move(site_for_cookies), std::move(first_auth_attempt), std::move(auth_info), std::move(resource_type), std::move(head), std::move(auth_challenge_responder));
}
void NetworkServiceClientInterceptorForTesting::OnCertificateRequested(uint32_t process_id, uint32_t routing_id, uint32_t request_id, ::network::mojom::blink::SSLCertRequestInfoPtr cert_info, OnCertificateRequestedCallback callback) {
  GetForwardingInterface()->OnCertificateRequested(std::move(process_id), std::move(routing_id), std::move(request_id), std::move(cert_info), std::move(callback));
}
void NetworkServiceClientInterceptorForTesting::OnSSLCertificateError(uint32_t process_id, uint32_t routing_id, uint32_t request_id, int32_t resource_type, const ::blink::KURL& url, ::network::mojom::blink::SSLInfoPtr ssl_info, bool fatal, OnSSLCertificateErrorCallback callback) {
  GetForwardingInterface()->OnSSLCertificateError(std::move(process_id), std::move(routing_id), std::move(request_id), std::move(resource_type), std::move(url), std::move(ssl_info), std::move(fatal), std::move(callback));
}
void NetworkServiceClientInterceptorForTesting::OnFileUploadRequested(uint32_t process_id, bool async, const WTF::Vector<base::FilePath>& file_paths, OnFileUploadRequestedCallback callback) {
  GetForwardingInterface()->OnFileUploadRequested(std::move(process_id), std::move(async), std::move(file_paths), std::move(callback));
}
void NetworkServiceClientInterceptorForTesting::OnCookieChange(int32_t process_id, int32_t routing_id, const ::blink::KURL& url, const ::blink::KURL& frame_url, const ::blink::WebCanonicalCookie& cookie, bool blocked_by_policy) {
  GetForwardingInterface()->OnCookieChange(std::move(process_id), std::move(routing_id), std::move(url), std::move(frame_url), std::move(cookie), std::move(blocked_by_policy));
}
void NetworkServiceClientInterceptorForTesting::OnCookiesRead(int32_t process_id, int32_t routing_id, const ::blink::KURL& url, const ::blink::KURL& frame_url, const WTF::Vector<::blink::WebCanonicalCookie>& cookie_list, bool blocked_by_policy) {
  GetForwardingInterface()->OnCookiesRead(std::move(process_id), std::move(routing_id), std::move(url), std::move(frame_url), std::move(cookie_list), std::move(blocked_by_policy));
}
NetworkServiceClientAsyncWaiter::NetworkServiceClientAsyncWaiter(
    NetworkServiceClient* proxy) : proxy_(proxy) {}

NetworkServiceClientAsyncWaiter::~NetworkServiceClientAsyncWaiter() = default;

void NetworkServiceClientAsyncWaiter::OnCertificateRequested(
    uint32_t process_id, uint32_t routing_id, uint32_t request_id, ::network::mojom::blink::SSLCertRequestInfoPtr cert_info, ::network::mojom::blink::X509CertificatePtr* out_x509_certificate, WTF::Vector<uint16_t>* out_algorithm_preferences, SSLPrivateKeyPtr* out_ssl_private_key, bool* out_cancel_certificate_selection) {
  base::RunLoop loop;
  proxy_->OnCertificateRequested(std::move(process_id),std::move(routing_id),std::move(request_id),std::move(cert_info),
      base::BindOnce(
          [](base::RunLoop* loop,
             ::network::mojom::blink::X509CertificatePtr* out_x509_certificate
,
             WTF::Vector<uint16_t>* out_algorithm_preferences
,
             SSLPrivateKeyPtr* out_ssl_private_key
,
             bool* out_cancel_certificate_selection
,
             ::network::mojom::blink::X509CertificatePtr x509_certificate,
             const WTF::Vector<uint16_t>& algorithm_preferences,
             SSLPrivateKeyPtr ssl_private_key,
             bool cancel_certificate_selection) {*out_x509_certificate = std::move(x509_certificate);*out_algorithm_preferences = std::move(algorithm_preferences);*out_ssl_private_key = std::move(ssl_private_key);*out_cancel_certificate_selection = std::move(cancel_certificate_selection);
            loop->Quit();
          },
          &loop,
          out_x509_certificate,
          out_algorithm_preferences,
          out_ssl_private_key,
          out_cancel_certificate_selection));
  loop.Run();
}
void NetworkServiceClientAsyncWaiter::OnSSLCertificateError(
    uint32_t process_id, uint32_t routing_id, uint32_t request_id, int32_t resource_type, const ::blink::KURL& url, ::network::mojom::blink::SSLInfoPtr ssl_info, bool fatal, int32_t* out_net_error) {
  base::RunLoop loop;
  proxy_->OnSSLCertificateError(std::move(process_id),std::move(routing_id),std::move(request_id),std::move(resource_type),std::move(url),std::move(ssl_info),std::move(fatal),
      base::BindOnce(
          [](base::RunLoop* loop,
             int32_t* out_net_error
,
             int32_t net_error) {*out_net_error = std::move(net_error);
            loop->Quit();
          },
          &loop,
          out_net_error));
  loop.Run();
}
void NetworkServiceClientAsyncWaiter::OnFileUploadRequested(
    uint32_t process_id, bool async, const WTF::Vector<base::FilePath>& file_paths, int32_t* out_net_error, WTF::Vector<base::File>* out_files) {
  base::RunLoop loop;
  proxy_->OnFileUploadRequested(std::move(process_id),std::move(async),std::move(file_paths),
      base::BindOnce(
          [](base::RunLoop* loop,
             int32_t* out_net_error
,
             WTF::Vector<base::File>* out_files
,
             int32_t net_error,
             WTF::Vector<base::File> files) {*out_net_error = std::move(net_error);*out_files = std::move(files);
            loop->Quit();
          },
          &loop,
          out_net_error,
          out_files));
  loop.Run();
}

const char NetworkService::Name_[] = "network.mojom.NetworkService";

class NetworkService_GetTotalNetworkUsages_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkService_GetTotalNetworkUsages_ForwardToCallback(
      NetworkService::GetTotalNetworkUsagesCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkService::GetTotalNetworkUsagesCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkService_GetTotalNetworkUsages_ForwardToCallback);
};

NetworkServiceProxy::NetworkServiceProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}
class NetworkServiceProxy_SetClient_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceProxy_SetClient_Message(
      uint32_t message_flags
      , NetworkServiceClientPtr param_client
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkService_SetClient_Name,
          message_flags)
      , param_client_(param_client.PassInterface()){}
  ~NetworkServiceProxy_SetClient_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      NetworkServiceClientPtr param_client) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceProxy_SetClient_Message>(
          kFlags
          , std::move(param_client)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkService_SetClient_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_SetClient_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::NetworkServiceClientPtrDataView>(
      param_client, &params->client, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid client in NetworkService.SetClient request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkService* impl) {
    impl->SetClient(
        NetworkServiceClientPtr(std::move(param_client_)));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkService_SetClient_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::NetworkServiceClientPtrDataView>(
      param_client_, &params->client, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid client in NetworkService.SetClient request");
  }
  NetworkServiceClientPtrInfo param_client_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceProxy_SetClient_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceProxy_SetClient_Message::kMessageTag = {};

void NetworkServiceProxy::SetClient(
    NetworkServiceClientPtr in_client) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkService::SetClient");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceProxy_SetClient_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_client));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkService::Name_);
  message.set_method_name("SetClient");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void NetworkServiceProxy::CreateNetworkContext(
    ::network::mojom::blink::NetworkContextRequest in_context, ::network::mojom::blink::NetworkContextParamsPtr in_params) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkService::CreateNetworkContext");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kNetworkService_CreateNetworkContext_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_CreateNetworkContext_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::NetworkContextRequestDataView>(
      in_context, &params->context, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->context),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid context in NetworkService.CreateNetworkContext request");
  typename decltype(params->params)::BaseType::BufferWriter
      params_writer;
  mojo::internal::Serialize<::network::mojom::NetworkContextParamsDataView>(
      in_params, buffer, &params_writer, &serialization_context);
  params->params.Set(
      params_writer.is_null() ? nullptr : params_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->params.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null params in NetworkService.CreateNetworkContext request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkService::Name_);
  message.set_method_name("CreateNetworkContext");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkServiceProxy_ConfigureStubHostResolver_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceProxy_ConfigureStubHostResolver_Message(
      uint32_t message_flags
      , bool param_stub_resolver_enabled
      , base::Optional<WTF::Vector<DnsOverHttpsServerPtr>> param_dns_over_https_servers
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkService_ConfigureStubHostResolver_Name,
          message_flags)
      , param_stub_resolver_enabled_(std::move(param_stub_resolver_enabled))
      , param_dns_over_https_servers_(std::move(param_dns_over_https_servers)){}
  ~NetworkServiceProxy_ConfigureStubHostResolver_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      bool param_stub_resolver_enabled, base::Optional<WTF::Vector<DnsOverHttpsServerPtr>> param_dns_over_https_servers) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceProxy_ConfigureStubHostResolver_Message>(
          kFlags
          , std::move(param_stub_resolver_enabled)
          , std::move(param_dns_over_https_servers)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkService_ConfigureStubHostResolver_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_ConfigureStubHostResolver_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->stub_resolver_enabled = param_stub_resolver_enabled;
  typename decltype(params->dns_over_https_servers)::BaseType::BufferWriter
      dns_over_https_servers_writer;
  const mojo::internal::ContainerValidateParams dns_over_https_servers_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::network::mojom::DnsOverHttpsServerDataView>>(
      param_dns_over_https_servers, buffer, &dns_over_https_servers_writer, &dns_over_https_servers_validate_params,
      &serialization_context);
  params->dns_over_https_servers.Set(
      dns_over_https_servers_writer.is_null() ? nullptr : dns_over_https_servers_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkService* impl) {
    impl->ConfigureStubHostResolver(
        std::move(param_stub_resolver_enabled_),
        std::move(param_dns_over_https_servers_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkService_ConfigureStubHostResolver_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->stub_resolver_enabled = param_stub_resolver_enabled_;
  typename decltype(params->dns_over_https_servers)::BaseType::BufferWriter
      dns_over_https_servers_writer;
  const mojo::internal::ContainerValidateParams dns_over_https_servers_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::network::mojom::DnsOverHttpsServerDataView>>(
      param_dns_over_https_servers_, buffer, &dns_over_https_servers_writer, &dns_over_https_servers_validate_params,
      serialization_context);
  params->dns_over_https_servers.Set(
      dns_over_https_servers_writer.is_null() ? nullptr : dns_over_https_servers_writer.data());
  }
  bool param_stub_resolver_enabled_;
  base::Optional<WTF::Vector<DnsOverHttpsServerPtr>> param_dns_over_https_servers_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceProxy_ConfigureStubHostResolver_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceProxy_ConfigureStubHostResolver_Message::kMessageTag = {};

void NetworkServiceProxy::ConfigureStubHostResolver(
    bool in_stub_resolver_enabled, base::Optional<WTF::Vector<DnsOverHttpsServerPtr>> in_dns_over_https_servers) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkService::ConfigureStubHostResolver");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceProxy_ConfigureStubHostResolver_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_stub_resolver_enabled), std::move(in_dns_over_https_servers));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkService::Name_);
  message.set_method_name("ConfigureStubHostResolver");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkServiceProxy_DisableQuic_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceProxy_DisableQuic_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkService_DisableQuic_Name,
          message_flags){}
  ~NetworkServiceProxy_DisableQuic_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceProxy_DisableQuic_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkService_DisableQuic_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_DisableQuic_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkService* impl) {
    impl->DisableQuic();
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkService_DisableQuic_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceProxy_DisableQuic_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceProxy_DisableQuic_Message::kMessageTag = {};

void NetworkServiceProxy::DisableQuic(
    ) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkService::DisableQuic");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceProxy_DisableQuic_Message::Build(
      kSerialize, kExpectsResponse, kIsSync);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkService::Name_);
  message.set_method_name("DisableQuic");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkServiceProxy_SetUpHttpAuth_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceProxy_SetUpHttpAuth_Message(
      uint32_t message_flags
      , HttpAuthStaticParamsPtr param_http_auth_static_params
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkService_SetUpHttpAuth_Name,
          message_flags)
      , param_http_auth_static_params_(std::move(param_http_auth_static_params)){}
  ~NetworkServiceProxy_SetUpHttpAuth_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      HttpAuthStaticParamsPtr param_http_auth_static_params) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceProxy_SetUpHttpAuth_Message>(
          kFlags
          , std::move(param_http_auth_static_params)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkService_SetUpHttpAuth_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_SetUpHttpAuth_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->http_auth_static_params)::BaseType::BufferWriter
      http_auth_static_params_writer;
  mojo::internal::Serialize<::network::mojom::HttpAuthStaticParamsDataView>(
      param_http_auth_static_params, buffer, &http_auth_static_params_writer, &serialization_context);
  params->http_auth_static_params.Set(
      http_auth_static_params_writer.is_null() ? nullptr : http_auth_static_params_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->http_auth_static_params.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null http_auth_static_params in NetworkService.SetUpHttpAuth request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkService* impl) {
    impl->SetUpHttpAuth(
        std::move(param_http_auth_static_params_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkService_SetUpHttpAuth_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->http_auth_static_params)::BaseType::BufferWriter
      http_auth_static_params_writer;
  mojo::internal::Serialize<::network::mojom::HttpAuthStaticParamsDataView>(
      param_http_auth_static_params_, buffer, &http_auth_static_params_writer, serialization_context);
  params->http_auth_static_params.Set(
      http_auth_static_params_writer.is_null() ? nullptr : http_auth_static_params_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->http_auth_static_params.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null http_auth_static_params in NetworkService.SetUpHttpAuth request");
  }
  HttpAuthStaticParamsPtr param_http_auth_static_params_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceProxy_SetUpHttpAuth_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceProxy_SetUpHttpAuth_Message::kMessageTag = {};

void NetworkServiceProxy::SetUpHttpAuth(
    HttpAuthStaticParamsPtr in_http_auth_static_params) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkService::SetUpHttpAuth");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceProxy_SetUpHttpAuth_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_http_auth_static_params));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkService::Name_);
  message.set_method_name("SetUpHttpAuth");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkServiceProxy_ConfigureHttpAuthPrefs_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceProxy_ConfigureHttpAuthPrefs_Message(
      uint32_t message_flags
      , HttpAuthDynamicParamsPtr param_http_auth_dynamic_params
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkService_ConfigureHttpAuthPrefs_Name,
          message_flags)
      , param_http_auth_dynamic_params_(std::move(param_http_auth_dynamic_params)){}
  ~NetworkServiceProxy_ConfigureHttpAuthPrefs_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      HttpAuthDynamicParamsPtr param_http_auth_dynamic_params) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceProxy_ConfigureHttpAuthPrefs_Message>(
          kFlags
          , std::move(param_http_auth_dynamic_params)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkService_ConfigureHttpAuthPrefs_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_ConfigureHttpAuthPrefs_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->http_auth_dynamic_params)::BaseType::BufferWriter
      http_auth_dynamic_params_writer;
  mojo::internal::Serialize<::network::mojom::HttpAuthDynamicParamsDataView>(
      param_http_auth_dynamic_params, buffer, &http_auth_dynamic_params_writer, &serialization_context);
  params->http_auth_dynamic_params.Set(
      http_auth_dynamic_params_writer.is_null() ? nullptr : http_auth_dynamic_params_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->http_auth_dynamic_params.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null http_auth_dynamic_params in NetworkService.ConfigureHttpAuthPrefs request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkService* impl) {
    impl->ConfigureHttpAuthPrefs(
        std::move(param_http_auth_dynamic_params_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkService_ConfigureHttpAuthPrefs_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->http_auth_dynamic_params)::BaseType::BufferWriter
      http_auth_dynamic_params_writer;
  mojo::internal::Serialize<::network::mojom::HttpAuthDynamicParamsDataView>(
      param_http_auth_dynamic_params_, buffer, &http_auth_dynamic_params_writer, serialization_context);
  params->http_auth_dynamic_params.Set(
      http_auth_dynamic_params_writer.is_null() ? nullptr : http_auth_dynamic_params_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->http_auth_dynamic_params.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null http_auth_dynamic_params in NetworkService.ConfigureHttpAuthPrefs request");
  }
  HttpAuthDynamicParamsPtr param_http_auth_dynamic_params_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceProxy_ConfigureHttpAuthPrefs_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceProxy_ConfigureHttpAuthPrefs_Message::kMessageTag = {};

void NetworkServiceProxy::ConfigureHttpAuthPrefs(
    HttpAuthDynamicParamsPtr in_http_auth_dynamic_params) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkService::ConfigureHttpAuthPrefs");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceProxy_ConfigureHttpAuthPrefs_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_http_auth_dynamic_params));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkService::Name_);
  message.set_method_name("ConfigureHttpAuthPrefs");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkServiceProxy_SetRawHeadersAccess_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceProxy_SetRawHeadersAccess_Message(
      uint32_t message_flags
      , uint32_t param_process_id
      , bool param_allow
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkService_SetRawHeadersAccess_Name,
          message_flags)
      , param_process_id_(std::move(param_process_id))
      , param_allow_(std::move(param_allow)){}
  ~NetworkServiceProxy_SetRawHeadersAccess_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      uint32_t param_process_id, bool param_allow) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceProxy_SetRawHeadersAccess_Message>(
          kFlags
          , std::move(param_process_id)
          , std::move(param_allow)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkService_SetRawHeadersAccess_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_SetRawHeadersAccess_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->process_id = param_process_id;
  params->allow = param_allow;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkService* impl) {
    impl->SetRawHeadersAccess(
        std::move(param_process_id_),
        std::move(param_allow_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkService_SetRawHeadersAccess_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->process_id = param_process_id_;
  params->allow = param_allow_;
  }
  uint32_t param_process_id_;
  bool param_allow_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceProxy_SetRawHeadersAccess_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceProxy_SetRawHeadersAccess_Message::kMessageTag = {};

void NetworkServiceProxy::SetRawHeadersAccess(
    uint32_t in_process_id, bool in_allow) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkService::SetRawHeadersAccess");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceProxy_SetRawHeadersAccess_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_process_id), std::move(in_allow));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkService::Name_);
  message.set_method_name("SetRawHeadersAccess");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkServiceProxy_GetNetworkChangeManager_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceProxy_GetNetworkChangeManager_Message(
      uint32_t message_flags
      , ::network::mojom::blink::NetworkChangeManagerRequest param_network_change_manager
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkService_GetNetworkChangeManager_Name,
          message_flags)
      , param_network_change_manager_(std::move(param_network_change_manager)){}
  ~NetworkServiceProxy_GetNetworkChangeManager_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::blink::NetworkChangeManagerRequest param_network_change_manager) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceProxy_GetNetworkChangeManager_Message>(
          kFlags
          , std::move(param_network_change_manager)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkService_GetNetworkChangeManager_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_GetNetworkChangeManager_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::NetworkChangeManagerRequestDataView>(
      param_network_change_manager, &params->network_change_manager, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->network_change_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid network_change_manager in NetworkService.GetNetworkChangeManager request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkService* impl) {
    impl->GetNetworkChangeManager(
        std::move(param_network_change_manager_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkService_GetNetworkChangeManager_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::NetworkChangeManagerRequestDataView>(
      param_network_change_manager_, &params->network_change_manager, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->network_change_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid network_change_manager in NetworkService.GetNetworkChangeManager request");
  }
  ::network::mojom::blink::NetworkChangeManagerRequest param_network_change_manager_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceProxy_GetNetworkChangeManager_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceProxy_GetNetworkChangeManager_Message::kMessageTag = {};

void NetworkServiceProxy::GetNetworkChangeManager(
    ::network::mojom::blink::NetworkChangeManagerRequest in_network_change_manager) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkService::GetNetworkChangeManager");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceProxy_GetNetworkChangeManager_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_network_change_manager));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkService::Name_);
  message.set_method_name("GetNetworkChangeManager");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkServiceProxy_GetNetworkQualityEstimatorManager_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceProxy_GetNetworkQualityEstimatorManager_Message(
      uint32_t message_flags
      , ::network::mojom::blink::NetworkQualityEstimatorManagerRequest param_network_quality_estimator_manager
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkService_GetNetworkQualityEstimatorManager_Name,
          message_flags)
      , param_network_quality_estimator_manager_(std::move(param_network_quality_estimator_manager)){}
  ~NetworkServiceProxy_GetNetworkQualityEstimatorManager_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::blink::NetworkQualityEstimatorManagerRequest param_network_quality_estimator_manager) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceProxy_GetNetworkQualityEstimatorManager_Message>(
          kFlags
          , std::move(param_network_quality_estimator_manager)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkService_GetNetworkQualityEstimatorManager_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_GetNetworkQualityEstimatorManager_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::NetworkQualityEstimatorManagerRequestDataView>(
      param_network_quality_estimator_manager, &params->network_quality_estimator_manager, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->network_quality_estimator_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid network_quality_estimator_manager in NetworkService.GetNetworkQualityEstimatorManager request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkService* impl) {
    impl->GetNetworkQualityEstimatorManager(
        std::move(param_network_quality_estimator_manager_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkService_GetNetworkQualityEstimatorManager_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::NetworkQualityEstimatorManagerRequestDataView>(
      param_network_quality_estimator_manager_, &params->network_quality_estimator_manager, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->network_quality_estimator_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid network_quality_estimator_manager in NetworkService.GetNetworkQualityEstimatorManager request");
  }
  ::network::mojom::blink::NetworkQualityEstimatorManagerRequest param_network_quality_estimator_manager_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceProxy_GetNetworkQualityEstimatorManager_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceProxy_GetNetworkQualityEstimatorManager_Message::kMessageTag = {};

void NetworkServiceProxy::GetNetworkQualityEstimatorManager(
    ::network::mojom::blink::NetworkQualityEstimatorManagerRequest in_network_quality_estimator_manager) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkService::GetNetworkQualityEstimatorManager");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceProxy_GetNetworkQualityEstimatorManager_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_network_quality_estimator_manager));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkService::Name_);
  message.set_method_name("GetNetworkQualityEstimatorManager");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkServiceProxy_GetTotalNetworkUsages_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceProxy_GetTotalNetworkUsages_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkService_GetTotalNetworkUsages_Name,
          message_flags){}
  ~NetworkServiceProxy_GetTotalNetworkUsages_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceProxy_GetTotalNetworkUsages_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkService_GetTotalNetworkUsages_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_GetTotalNetworkUsages_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkService* impl, NetworkService::GetTotalNetworkUsagesCallback callback) {
    impl->GetTotalNetworkUsages(std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkService_GetTotalNetworkUsages_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceProxy_GetTotalNetworkUsages_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceProxy_GetTotalNetworkUsages_Message::kMessageTag = {};

void NetworkServiceProxy::GetTotalNetworkUsages(
    GetTotalNetworkUsagesCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkService::GetTotalNetworkUsages");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceProxy_GetTotalNetworkUsages_Message::Build(
      kSerialize, kExpectsResponse, kIsSync);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkService::Name_);
  message.set_method_name("GetTotalNetworkUsages");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkService_GetTotalNetworkUsages_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkServiceProxy_UpdateSignedTreeHead_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceProxy_UpdateSignedTreeHead_Message(
      uint32_t message_flags
      , ::network::mojom::blink::SignedTreeHeadPtr param_signed_tree_head
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkService_UpdateSignedTreeHead_Name,
          message_flags)
      , param_signed_tree_head_(std::move(param_signed_tree_head)){}
  ~NetworkServiceProxy_UpdateSignedTreeHead_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::blink::SignedTreeHeadPtr param_signed_tree_head) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceProxy_UpdateSignedTreeHead_Message>(
          kFlags
          , std::move(param_signed_tree_head)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkService_UpdateSignedTreeHead_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_UpdateSignedTreeHead_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->signed_tree_head)::BaseType::BufferWriter
      signed_tree_head_writer;
  mojo::internal::Serialize<::network::mojom::SignedTreeHeadDataView>(
      param_signed_tree_head, buffer, &signed_tree_head_writer, &serialization_context);
  params->signed_tree_head.Set(
      signed_tree_head_writer.is_null() ? nullptr : signed_tree_head_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->signed_tree_head.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null signed_tree_head in NetworkService.UpdateSignedTreeHead request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkService* impl) {
    impl->UpdateSignedTreeHead(
        std::move(param_signed_tree_head_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkService_UpdateSignedTreeHead_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->signed_tree_head)::BaseType::BufferWriter
      signed_tree_head_writer;
  mojo::internal::Serialize<::network::mojom::SignedTreeHeadDataView>(
      param_signed_tree_head_, buffer, &signed_tree_head_writer, serialization_context);
  params->signed_tree_head.Set(
      signed_tree_head_writer.is_null() ? nullptr : signed_tree_head_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->signed_tree_head.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null signed_tree_head in NetworkService.UpdateSignedTreeHead request");
  }
  ::network::mojom::blink::SignedTreeHeadPtr param_signed_tree_head_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceProxy_UpdateSignedTreeHead_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceProxy_UpdateSignedTreeHead_Message::kMessageTag = {};

void NetworkServiceProxy::UpdateSignedTreeHead(
    ::network::mojom::blink::SignedTreeHeadPtr in_signed_tree_head) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkService::UpdateSignedTreeHead");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceProxy_UpdateSignedTreeHead_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_signed_tree_head));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkService::Name_);
  message.set_method_name("UpdateSignedTreeHead");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkServiceProxy_SetCryptConfig_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceProxy_SetCryptConfig_Message(
      uint32_t message_flags
      , CryptConfigPtr param_crypt_config
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkService_SetCryptConfig_Name,
          message_flags)
      , param_crypt_config_(std::move(param_crypt_config)){}
  ~NetworkServiceProxy_SetCryptConfig_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      CryptConfigPtr param_crypt_config) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceProxy_SetCryptConfig_Message>(
          kFlags
          , std::move(param_crypt_config)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkService_SetCryptConfig_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_SetCryptConfig_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->crypt_config)::BaseType::BufferWriter
      crypt_config_writer;
  mojo::internal::Serialize<::network::mojom::CryptConfigDataView>(
      param_crypt_config, buffer, &crypt_config_writer, &serialization_context);
  params->crypt_config.Set(
      crypt_config_writer.is_null() ? nullptr : crypt_config_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->crypt_config.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null crypt_config in NetworkService.SetCryptConfig request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkService* impl) {
    impl->SetCryptConfig(
        std::move(param_crypt_config_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkService_SetCryptConfig_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->crypt_config)::BaseType::BufferWriter
      crypt_config_writer;
  mojo::internal::Serialize<::network::mojom::CryptConfigDataView>(
      param_crypt_config_, buffer, &crypt_config_writer, serialization_context);
  params->crypt_config.Set(
      crypt_config_writer.is_null() ? nullptr : crypt_config_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->crypt_config.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null crypt_config in NetworkService.SetCryptConfig request");
  }
  CryptConfigPtr param_crypt_config_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceProxy_SetCryptConfig_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceProxy_SetCryptConfig_Message::kMessageTag = {};

void NetworkServiceProxy::SetCryptConfig(
    CryptConfigPtr in_crypt_config) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkService::SetCryptConfig");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceProxy_SetCryptConfig_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_crypt_config));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkService::Name_);
  message.set_method_name("SetCryptConfig");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkService_GetTotalNetworkUsages_ProxyToResponder {
 public:
  static NetworkService::GetTotalNetworkUsagesCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkService_GetTotalNetworkUsages_ProxyToResponder> proxy(
        new NetworkService_GetTotalNetworkUsages_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkService_GetTotalNetworkUsages_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkService_GetTotalNetworkUsages_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkService_GetTotalNetworkUsages_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkService::GetTotalNetworkUsagesCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      WTF::Vector<::network::mojom::blink::NetworkUsagePtr> in_total_network_usages);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkService_GetTotalNetworkUsages_ProxyToResponder);
};
class NetworkService_GetTotalNetworkUsages_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkService_GetTotalNetworkUsages_Response_Message(
      uint32_t message_flags
      , WTF::Vector<::network::mojom::blink::NetworkUsagePtr> param_total_network_usages
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkService_GetTotalNetworkUsages_Name,
          message_flags)
      , param_total_network_usages_(std::move(param_total_network_usages)){}
  ~NetworkService_GetTotalNetworkUsages_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      WTF::Vector<::network::mojom::blink::NetworkUsagePtr> param_total_network_usages) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkService_GetTotalNetworkUsages_Response_Message>(
          kFlags
          , std::move(param_total_network_usages)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkService_GetTotalNetworkUsages_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_GetTotalNetworkUsages_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->total_network_usages)::BaseType::BufferWriter
      total_network_usages_writer;
  const mojo::internal::ContainerValidateParams total_network_usages_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::network::mojom::NetworkUsageDataView>>(
      param_total_network_usages, buffer, &total_network_usages_writer, &total_network_usages_validate_params,
      &serialization_context);
  params->total_network_usages.Set(
      total_network_usages_writer.is_null() ? nullptr : total_network_usages_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->total_network_usages.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null total_network_usages in NetworkService.GetTotalNetworkUsages response");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkService::GetTotalNetworkUsagesCallback* callback) {
    std::move(*callback).Run(
        std::move(param_total_network_usages_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkService_GetTotalNetworkUsages_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->total_network_usages)::BaseType::BufferWriter
      total_network_usages_writer;
  const mojo::internal::ContainerValidateParams total_network_usages_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::network::mojom::NetworkUsageDataView>>(
      param_total_network_usages_, buffer, &total_network_usages_writer, &total_network_usages_validate_params,
      serialization_context);
  params->total_network_usages.Set(
      total_network_usages_writer.is_null() ? nullptr : total_network_usages_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->total_network_usages.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null total_network_usages in NetworkService.GetTotalNetworkUsages response");
  }
  WTF::Vector<::network::mojom::blink::NetworkUsagePtr> param_total_network_usages_;

  DISALLOW_COPY_AND_ASSIGN(NetworkService_GetTotalNetworkUsages_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkService_GetTotalNetworkUsages_Response_Message::kMessageTag = {};

bool NetworkService_GetTotalNetworkUsages_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkService::GetTotalNetworkUsagesCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkService_GetTotalNetworkUsages_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkService_GetTotalNetworkUsages_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkService_GetTotalNetworkUsages_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  WTF::Vector<::network::mojom::blink::NetworkUsagePtr> p_total_network_usages{};
  NetworkService_GetTotalNetworkUsages_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadTotalNetworkUsages(&p_total_network_usages))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkService::GetTotalNetworkUsages response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_total_network_usages));
  return true;
}

void NetworkService_GetTotalNetworkUsages_ProxyToResponder::Run(
    WTF::Vector<::network::mojom::blink::NetworkUsagePtr> in_total_network_usages) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkService_GetTotalNetworkUsages_Response_Message::Build(kSerialize, is_sync_, std::move(in_total_network_usages));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::GetTotalNetworkUsagesCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkService::Name_);
  message.set_method_name("GetTotalNetworkUsages");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool NetworkServiceStubDispatch::Accept(
    NetworkService* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kNetworkService_SetClient_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::SetClient",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceProxy_SetClient_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkService_SetClient_Params_Data* params =
          reinterpret_cast<internal::NetworkService_SetClient_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      NetworkServiceClientPtr p_client{};
      NetworkService_SetClient_ParamsDataView input_data_view(params, &serialization_context);
      
      p_client =
          input_data_view.TakeClient<decltype(p_client)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkService::SetClient deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetClient(
std::move(p_client));
      return true;
    }
    case internal::kNetworkService_CreateNetworkContext_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::CreateNetworkContext",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::NetworkService_CreateNetworkContext_Params_Data* params =
          reinterpret_cast<internal::NetworkService_CreateNetworkContext_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::blink::NetworkContextRequest p_context{};
      ::network::mojom::blink::NetworkContextParamsPtr p_params{};
      NetworkService_CreateNetworkContext_ParamsDataView input_data_view(params, &serialization_context);
      
      p_context =
          input_data_view.TakeContext<decltype(p_context)>();
      if (!input_data_view.ReadParams(&p_params))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkService::CreateNetworkContext deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateNetworkContext(
std::move(p_context), 
std::move(p_params));
      return true;
    }
    case internal::kNetworkService_ConfigureStubHostResolver_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::ConfigureStubHostResolver",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceProxy_ConfigureStubHostResolver_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkService_ConfigureStubHostResolver_Params_Data* params =
          reinterpret_cast<internal::NetworkService_ConfigureStubHostResolver_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      bool p_stub_resolver_enabled{};
      base::Optional<WTF::Vector<DnsOverHttpsServerPtr>> p_dns_over_https_servers{};
      NetworkService_ConfigureStubHostResolver_ParamsDataView input_data_view(params, &serialization_context);
      
      p_stub_resolver_enabled = input_data_view.stub_resolver_enabled();
      if (!input_data_view.ReadDnsOverHttpsServers(&p_dns_over_https_servers))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkService::ConfigureStubHostResolver deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ConfigureStubHostResolver(
std::move(p_stub_resolver_enabled), 
std::move(p_dns_over_https_servers));
      return true;
    }
    case internal::kNetworkService_DisableQuic_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::DisableQuic",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceProxy_DisableQuic_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkService_DisableQuic_Params_Data* params =
          reinterpret_cast<internal::NetworkService_DisableQuic_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      NetworkService_DisableQuic_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkService::DisableQuic deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->DisableQuic();
      return true;
    }
    case internal::kNetworkService_SetUpHttpAuth_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::SetUpHttpAuth",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceProxy_SetUpHttpAuth_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkService_SetUpHttpAuth_Params_Data* params =
          reinterpret_cast<internal::NetworkService_SetUpHttpAuth_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      HttpAuthStaticParamsPtr p_http_auth_static_params{};
      NetworkService_SetUpHttpAuth_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadHttpAuthStaticParams(&p_http_auth_static_params))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkService::SetUpHttpAuth deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetUpHttpAuth(
std::move(p_http_auth_static_params));
      return true;
    }
    case internal::kNetworkService_ConfigureHttpAuthPrefs_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::ConfigureHttpAuthPrefs",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceProxy_ConfigureHttpAuthPrefs_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkService_ConfigureHttpAuthPrefs_Params_Data* params =
          reinterpret_cast<internal::NetworkService_ConfigureHttpAuthPrefs_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      HttpAuthDynamicParamsPtr p_http_auth_dynamic_params{};
      NetworkService_ConfigureHttpAuthPrefs_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadHttpAuthDynamicParams(&p_http_auth_dynamic_params))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkService::ConfigureHttpAuthPrefs deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ConfigureHttpAuthPrefs(
std::move(p_http_auth_dynamic_params));
      return true;
    }
    case internal::kNetworkService_SetRawHeadersAccess_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::SetRawHeadersAccess",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceProxy_SetRawHeadersAccess_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkService_SetRawHeadersAccess_Params_Data* params =
          reinterpret_cast<internal::NetworkService_SetRawHeadersAccess_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_process_id{};
      bool p_allow{};
      NetworkService_SetRawHeadersAccess_ParamsDataView input_data_view(params, &serialization_context);
      
      p_process_id = input_data_view.process_id();
      p_allow = input_data_view.allow();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkService::SetRawHeadersAccess deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetRawHeadersAccess(
std::move(p_process_id), 
std::move(p_allow));
      return true;
    }
    case internal::kNetworkService_GetNetworkChangeManager_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::GetNetworkChangeManager",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceProxy_GetNetworkChangeManager_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkService_GetNetworkChangeManager_Params_Data* params =
          reinterpret_cast<internal::NetworkService_GetNetworkChangeManager_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::blink::NetworkChangeManagerRequest p_network_change_manager{};
      NetworkService_GetNetworkChangeManager_ParamsDataView input_data_view(params, &serialization_context);
      
      p_network_change_manager =
          input_data_view.TakeNetworkChangeManager<decltype(p_network_change_manager)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkService::GetNetworkChangeManager deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetNetworkChangeManager(
std::move(p_network_change_manager));
      return true;
    }
    case internal::kNetworkService_GetNetworkQualityEstimatorManager_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::GetNetworkQualityEstimatorManager",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceProxy_GetNetworkQualityEstimatorManager_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkService_GetNetworkQualityEstimatorManager_Params_Data* params =
          reinterpret_cast<internal::NetworkService_GetNetworkQualityEstimatorManager_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::blink::NetworkQualityEstimatorManagerRequest p_network_quality_estimator_manager{};
      NetworkService_GetNetworkQualityEstimatorManager_ParamsDataView input_data_view(params, &serialization_context);
      
      p_network_quality_estimator_manager =
          input_data_view.TakeNetworkQualityEstimatorManager<decltype(p_network_quality_estimator_manager)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkService::GetNetworkQualityEstimatorManager deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetNetworkQualityEstimatorManager(
std::move(p_network_quality_estimator_manager));
      return true;
    }
    case internal::kNetworkService_GetTotalNetworkUsages_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::GetTotalNetworkUsages",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkService_UpdateSignedTreeHead_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::UpdateSignedTreeHead",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceProxy_UpdateSignedTreeHead_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkService_UpdateSignedTreeHead_Params_Data* params =
          reinterpret_cast<internal::NetworkService_UpdateSignedTreeHead_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::blink::SignedTreeHeadPtr p_signed_tree_head{};
      NetworkService_UpdateSignedTreeHead_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadSignedTreeHead(&p_signed_tree_head))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkService::UpdateSignedTreeHead deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->UpdateSignedTreeHead(
std::move(p_signed_tree_head));
      return true;
    }
    case internal::kNetworkService_SetCryptConfig_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::SetCryptConfig",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceProxy_SetCryptConfig_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkService_SetCryptConfig_Params_Data* params =
          reinterpret_cast<internal::NetworkService_SetCryptConfig_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      CryptConfigPtr p_crypt_config{};
      NetworkService_SetCryptConfig_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadCryptConfig(&p_crypt_config))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkService::SetCryptConfig deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetCryptConfig(
std::move(p_crypt_config));
      return true;
    }
  }
  return false;
}

// static
bool NetworkServiceStubDispatch::AcceptWithResponder(
    NetworkService* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kNetworkService_SetClient_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::SetClient",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkService_CreateNetworkContext_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::CreateNetworkContext",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkService_ConfigureStubHostResolver_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::ConfigureStubHostResolver",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkService_DisableQuic_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::DisableQuic",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkService_SetUpHttpAuth_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::SetUpHttpAuth",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkService_ConfigureHttpAuthPrefs_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::ConfigureHttpAuthPrefs",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkService_SetRawHeadersAccess_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::SetRawHeadersAccess",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkService_GetNetworkChangeManager_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::GetNetworkChangeManager",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkService_GetNetworkQualityEstimatorManager_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::GetNetworkQualityEstimatorManager",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkService_GetTotalNetworkUsages_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::GetTotalNetworkUsages",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceProxy_GetTotalNetworkUsages_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkService::GetTotalNetworkUsagesCallback callback =
              NetworkService_GetTotalNetworkUsages_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkService_GetTotalNetworkUsages_Params_Data* params =
          reinterpret_cast<
              internal::NetworkService_GetTotalNetworkUsages_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      NetworkService_GetTotalNetworkUsages_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkService::GetTotalNetworkUsages deserializer");
        return false;
      }
      NetworkService::GetTotalNetworkUsagesCallback callback =
          NetworkService_GetTotalNetworkUsages_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetTotalNetworkUsages(std::move(callback));
      return true;
    }
    case internal::kNetworkService_UpdateSignedTreeHead_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::UpdateSignedTreeHead",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkService_SetCryptConfig_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::SetCryptConfig",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool NetworkServiceRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "NetworkService RequestValidator");

  switch (message->header()->name) {
    case internal::kNetworkService_SetClient_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_SetClient_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkService_CreateNetworkContext_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_CreateNetworkContext_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkService_ConfigureStubHostResolver_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_ConfigureStubHostResolver_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkService_DisableQuic_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_DisableQuic_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkService_SetUpHttpAuth_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_SetUpHttpAuth_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkService_ConfigureHttpAuthPrefs_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_ConfigureHttpAuthPrefs_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkService_SetRawHeadersAccess_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_SetRawHeadersAccess_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkService_GetNetworkChangeManager_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_GetNetworkChangeManager_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkService_GetNetworkQualityEstimatorManager_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_GetNetworkQualityEstimatorManager_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkService_GetTotalNetworkUsages_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_GetTotalNetworkUsages_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkService_UpdateSignedTreeHead_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_UpdateSignedTreeHead_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkService_SetCryptConfig_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_SetCryptConfig_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool NetworkServiceResponseValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "NetworkService ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kNetworkService_GetTotalNetworkUsages_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_GetTotalNetworkUsages_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
void NetworkServiceInterceptorForTesting::SetClient(NetworkServiceClientPtr client) {
  GetForwardingInterface()->SetClient(std::move(client));
}
void NetworkServiceInterceptorForTesting::CreateNetworkContext(::network::mojom::blink::NetworkContextRequest context, ::network::mojom::blink::NetworkContextParamsPtr params) {
  GetForwardingInterface()->CreateNetworkContext(std::move(context), std::move(params));
}
void NetworkServiceInterceptorForTesting::ConfigureStubHostResolver(bool stub_resolver_enabled, base::Optional<WTF::Vector<DnsOverHttpsServerPtr>> dns_over_https_servers) {
  GetForwardingInterface()->ConfigureStubHostResolver(std::move(stub_resolver_enabled), std::move(dns_over_https_servers));
}
void NetworkServiceInterceptorForTesting::DisableQuic() {
  GetForwardingInterface()->DisableQuic();
}
void NetworkServiceInterceptorForTesting::SetUpHttpAuth(HttpAuthStaticParamsPtr http_auth_static_params) {
  GetForwardingInterface()->SetUpHttpAuth(std::move(http_auth_static_params));
}
void NetworkServiceInterceptorForTesting::ConfigureHttpAuthPrefs(HttpAuthDynamicParamsPtr http_auth_dynamic_params) {
  GetForwardingInterface()->ConfigureHttpAuthPrefs(std::move(http_auth_dynamic_params));
}
void NetworkServiceInterceptorForTesting::SetRawHeadersAccess(uint32_t process_id, bool allow) {
  GetForwardingInterface()->SetRawHeadersAccess(std::move(process_id), std::move(allow));
}
void NetworkServiceInterceptorForTesting::GetNetworkChangeManager(::network::mojom::blink::NetworkChangeManagerRequest network_change_manager) {
  GetForwardingInterface()->GetNetworkChangeManager(std::move(network_change_manager));
}
void NetworkServiceInterceptorForTesting::GetNetworkQualityEstimatorManager(::network::mojom::blink::NetworkQualityEstimatorManagerRequest network_quality_estimator_manager) {
  GetForwardingInterface()->GetNetworkQualityEstimatorManager(std::move(network_quality_estimator_manager));
}
void NetworkServiceInterceptorForTesting::GetTotalNetworkUsages(GetTotalNetworkUsagesCallback callback) {
  GetForwardingInterface()->GetTotalNetworkUsages(std::move(callback));
}
void NetworkServiceInterceptorForTesting::UpdateSignedTreeHead(::network::mojom::blink::SignedTreeHeadPtr signed_tree_head) {
  GetForwardingInterface()->UpdateSignedTreeHead(std::move(signed_tree_head));
}
void NetworkServiceInterceptorForTesting::SetCryptConfig(CryptConfigPtr crypt_config) {
  GetForwardingInterface()->SetCryptConfig(std::move(crypt_config));
}
NetworkServiceAsyncWaiter::NetworkServiceAsyncWaiter(
    NetworkService* proxy) : proxy_(proxy) {}

NetworkServiceAsyncWaiter::~NetworkServiceAsyncWaiter() = default;

void NetworkServiceAsyncWaiter::GetTotalNetworkUsages(
    WTF::Vector<::network::mojom::blink::NetworkUsagePtr>* out_total_network_usages) {
  base::RunLoop loop;
  proxy_->GetTotalNetworkUsages(
      base::BindOnce(
          [](base::RunLoop* loop,
             WTF::Vector<::network::mojom::blink::NetworkUsagePtr>* out_total_network_usages
,
             WTF::Vector<::network::mojom::blink::NetworkUsagePtr> total_network_usages) {*out_total_network_usages = std::move(total_network_usages);
            loop->Quit();
          },
          &loop,
          out_total_network_usages));
  loop.Run();
}

}  // namespace blink
}  // namespace mojom
}  // namespace network

namespace mojo {


// static
bool StructTraits<::network::mojom::blink::DnsOverHttpsServer::DataView, ::network::mojom::blink::DnsOverHttpsServerPtr>::Read(
    ::network::mojom::blink::DnsOverHttpsServer::DataView input,
    ::network::mojom::blink::DnsOverHttpsServerPtr* output) {
  bool success = true;
  ::network::mojom::blink::DnsOverHttpsServerPtr result(::network::mojom::blink::DnsOverHttpsServer::New());
  
      if (!input.ReadUrl(&result->url))
        success = false;
      result->use_posts = input.use_posts();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::network::mojom::blink::HttpAuthStaticParams::DataView, ::network::mojom::blink::HttpAuthStaticParamsPtr>::Read(
    ::network::mojom::blink::HttpAuthStaticParams::DataView input,
    ::network::mojom::blink::HttpAuthStaticParamsPtr* output) {
  bool success = true;
  ::network::mojom::blink::HttpAuthStaticParamsPtr result(::network::mojom::blink::HttpAuthStaticParams::New());
  
      if (!input.ReadSupportedSchemes(&result->supported_schemes))
        success = false;
      if (!input.ReadGssapiLibraryName(&result->gssapi_library_name))
        success = false;
      result->allow_gssapi_library_load = input.allow_gssapi_library_load();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::network::mojom::blink::HttpAuthDynamicParams::DataView, ::network::mojom::blink::HttpAuthDynamicParamsPtr>::Read(
    ::network::mojom::blink::HttpAuthDynamicParams::DataView input,
    ::network::mojom::blink::HttpAuthDynamicParamsPtr* output) {
  bool success = true;
  ::network::mojom::blink::HttpAuthDynamicParamsPtr result(::network::mojom::blink::HttpAuthDynamicParams::New());
  
      if (!input.ReadServerWhitelist(&result->server_whitelist))
        success = false;
      if (!input.ReadDelegateWhitelist(&result->delegate_whitelist))
        success = false;
      result->negotiate_disable_cname_lookup = input.negotiate_disable_cname_lookup();
      result->enable_negotiate_port = input.enable_negotiate_port();
      result->ntlm_v2_enabled = input.ntlm_v2_enabled();
      if (!input.ReadAndroidNegotiateAccountType(&result->android_negotiate_account_type))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::network::mojom::blink::CryptConfig::DataView, ::network::mojom::blink::CryptConfigPtr>::Read(
    ::network::mojom::blink::CryptConfig::DataView input,
    ::network::mojom::blink::CryptConfigPtr* output) {
  bool success = true;
  ::network::mojom::blink::CryptConfigPtr result(::network::mojom::blink::CryptConfig::New());
  
      if (!input.ReadStore(&result->store))
        success = false;
      if (!input.ReadProductName(&result->product_name))
        success = false;
      result->should_use_preference = input.should_use_preference();
      if (!input.ReadUserDataPath(&result->user_data_path))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif